from typing import Any, List, Optional, NamedTuple

from .model import (
    URL,
    Role,
    User,
    Guild,
    Channel,
    UserChat,
    RolesReturn,
    GuildsReturn,
    ReactionUser,
    DirectMessage,
    InvitesReturn,
    ChannelMessage,
    ChannelsReturn,
    ChannelRoleInfo,
    GuildRoleReturn,
    UserChatsReturn,
    BlackListsReturn,
    GuildBoostReturn,
    GuildUsersReturn,
    ChannelRoleReturn,
    GuildEmojisReturn,
    IntimacyIndexReturn,
    MessageCreateReturn,
    DirectMessagesReturn,
    ChannelMessagesReturn,
    ChannelRoleSyncResult,
    GetUserJoinedChannelReturn,
)


class ApiMethod(NamedTuple):
    method: str
    restype: Optional[type] = None


api_method_map = {
    "asset/create": ApiMethod("POST", URL),
    "blacklist/create": ApiMethod("POST", None),
    "blacklist/delete": ApiMethod("POST", None),
    "blacklist/list": ApiMethod("GET", BlackListsReturn),
    "channel-user/get-joined-channel": ApiMethod("GET", GetUserJoinedChannelReturn),
    "channel-role/create": ApiMethod("POST", ChannelRoleReturn),
    "channel-role/delete": ApiMethod("POST", None),
    "channel-role/index": ApiMethod("GET", ChannelRoleInfo),
    "channel-role/update": ApiMethod("POST", ChannelRoleReturn),
    "channel-role/sync": ApiMethod("POST", ChannelRoleSyncResult),
    "channel/create": ApiMethod("POST", Channel),
    "channel/delete": ApiMethod("POST", None),
    "channel/update": ApiMethod("POST", Channel),
    "channel/list": ApiMethod("GET", ChannelsReturn),
    "channel/move-user": ApiMethod("POST", None),
    "channel/user-list": ApiMethod("GET", List[User]),
    "channel/view": ApiMethod("GET", Channel),
    "direct-message/add-reaction": ApiMethod("POST", None),
    "direct-message/create": ApiMethod("POST", MessageCreateReturn),
    "direct-message/delete": ApiMethod("POST", None),
    "direct-message/delete-reaction": ApiMethod("POST", None),
    "direct-message/list": ApiMethod("GET", DirectMessagesReturn),
    "direct-message/reaction-list": ApiMethod("GET", List[ReactionUser]),
    "direct-message/update": ApiMethod("POST", None),
    "direct-message/view": ApiMethod("GET", DirectMessage),
    "gateway/index": ApiMethod("GET", URL),
    "guild-boost/history": ApiMethod("GET", GuildBoostReturn),
    "guild-emoji/create": ApiMethod("POST", None),
    "guild-emoji/delete": ApiMethod("POST", None),
    "guild-emoji/list": ApiMethod("GET", GuildEmojisReturn),
    "guild-emoji/update": ApiMethod("POST", None),
    "guild-mute/create": ApiMethod("POST", None),
    "guild-mute/delete": ApiMethod("POST", None),
    "guild-mute/list": ApiMethod("GET", None),
    "guild-role/create": ApiMethod("POST", Role),
    "guild-role/delete": ApiMethod("POST", None),
    "guild-role/grant": ApiMethod("POST", GuildRoleReturn),
    "guild-role/list": ApiMethod("GET", RolesReturn),
    "guild-role/revoke": ApiMethod("POST", GuildRoleReturn),
    "guild-role/update": ApiMethod("POST", Role),
    "guild/kickout": ApiMethod("POST", None),
    "guild/leave": ApiMethod("POST", None),
    "guild/list": ApiMethod("GET", GuildsReturn),
    "guild/nickname": ApiMethod("POST", None),
    "guild/user-list": ApiMethod("GET", GuildUsersReturn),
    "guild/view": ApiMethod("GET", Guild),
    "intimacy/index": ApiMethod("GET", IntimacyIndexReturn),
    "intimacy/update": ApiMethod("POST", None),
    "invite/create": ApiMethod("POST", URL),
    "invite/delete": ApiMethod("POST", None),
    "invite/list": ApiMethod("GET", InvitesReturn),
    "message/add-reaction": ApiMethod("POST", None),
    "message/create": ApiMethod("POST", MessageCreateReturn),
    "message/delete": ApiMethod("POST", None),
    "message/delete-reaction": ApiMethod("POST", None),
    "message/list": ApiMethod("GET", ChannelMessagesReturn),
    "message/reaction-list": ApiMethod("GET", List[ReactionUser]),
    "message/update": ApiMethod("POST", None),
    "message/view": ApiMethod("GET", ChannelMessage),
    "user-chat/create": ApiMethod("POST", UserChat),
    "user-chat/delete": ApiMethod("POST", None),
    "user-chat/list": ApiMethod("GET", UserChatsReturn),
    "user-chat/view": ApiMethod("GET", UserChat),
    "user/me": ApiMethod("GET", User),
    "user/offline": ApiMethod("POST", None),
    "user/view": ApiMethod("GET", User),
}


def get_api_method(api: str) -> str:
    if api not in api_method_map:
        return "POST"
    return api_method_map[api].method


def get_api_restype(api: str) -> Any:
    if api not in api_method_map:
        return None
    return api_method_map[api].restype



import inspect
from enum import IntEnum
from typing import List, Type, Union, Optional
from typing_extensions import Literal, override

from pygtrie import StringTrie
from nonebot.utils import escape_tag
from nonebot.compat import model_dump
from pydantic import Field, HttpUrl, BaseModel, validator

from nonebot.adapters import Event as BaseEvent

from .utils import AttrDict
from .compat import model_validator
from .exception import NoLogException
from .message import Message, MessageDeserializer
from .api import Role, User, Emoji, Guild, Channel


class EventTypes(IntEnum):
    """
    事件主要格式
    Kaiheila 协议事件，字段与 Kaiheila 一致。各事件字段参考 `Kaiheila 文档`

    .. Kaiheila 文档:
        https://developer.kaiheila.cn/doc/event/event-introduction#事件主要格式
    """

    text = 1
    image = 2
    video = 3
    file = 4
    audio = 8
    kmarkdown = 9
    card = 10
    sys = 255


class SignalTypes(IntEnum):
    """
    信令类型
    Kaiheila 协议信令，字段与 Kaiheila 一致。各事件字段参考 `Kaiheila 文档`

    .. Kaiheila 文档:
        https://developer.kaiheila.cn/doc/websocket#信令格式
    """

    EVENT = 0
    HELLO = 1
    PING = 2
    PONG = 3
    RESUME = 4
    RECONNECT = 5
    RESUME_ACK = 6
    SYS = 255


class Attachment(BaseModel):
    type: str
    name: str
    url: HttpUrl
    file_type: Optional[str] = Field(None)
    size: Optional[int] = Field(None)
    duration: Optional[float] = Field(None)
    width: Optional[int] = Field(None)
    hight: Optional[int] = Field(None)


class Extra(BaseModel):
    type_: Union[int, str] = Field(None, alias="type")
    guild_id: Optional[str] = Field(None)
    channel_name: Optional[str] = Field(None)
    mention: Optional[List[str]] = Field(None)
    mention_all: Optional[bool] = Field(None)
    mention_roles: Optional[List[str]] = Field(None)
    mention_here: Optional[bool] = Field(None)
    author: Optional[User] = Field(None)
    body: Optional[AttrDict] = Field(None)
    attachments: Optional[Attachment] = Field(None)
    code: Optional[str] = Field(None)

    @validator("body", pre=True)
    def convert_body(cls, v):
        if v is None:
            return None

        if not isinstance(v, dict):
            raise TypeError("body must be dict")
        if not isinstance(v, AttrDict):
            v = AttrDict(v)
        return v

    class Config:
        arbitrary_types_allowed = True


class OriginEvent(BaseEvent):
    """为了区分信令中非Event事件，增加了前置OriginEvent"""

    __event__ = ""

    post_type: str

    @override
    def get_type(self) -> str:
        return self.post_type

    @override
    def get_event_name(self) -> str:
        return self.post_type

    @override
    def get_event_description(self) -> str:
        return escape_tag(str(model_dump(self)))

    @override
    def get_message(self) -> Message:
        raise ValueError("Event has no message!")

    @override
    def get_plaintext(self) -> str:
        raise ValueError("Event has no message!")

    @override
    def get_user_id(self) -> str:
        raise ValueError("Event has no message!")

    @override
    def get_session_id(self) -> str:
        raise ValueError("Event has no message!")

    @override
    def is_tome(self) -> bool:
        return False


class Kmarkdown(BaseModel):
    raw_content: str
    mention_part: list
    mention_role_part: list


class EventMessage(BaseModel):
    type: Union[int, str]
    guild_id: Optional[str] = None
    channel_name: Optional[str] = None
    mention: Optional[List] = None
    mention_all: Optional[bool] = None
    mention_roles: Optional[List] = None
    mention_here: Optional[bool] = None
    nav_channels: Optional[List] = None
    author: User

    kmarkdown: Optional[Kmarkdown] = None

    code: Optional[str] = None
    attachments: Optional[Attachment] = None

    content: Message

    @model_validator(mode="before")
    def parse_message(cls, values: dict):
        values["content"] = MessageDeserializer(
            values["type"],
            values,
        ).deserialize()
        return values


class Event(OriginEvent):
    """
    事件主要格式，来自 d 字段
    Kaiheila 协议事件，字段与 Kaiheila 一致。各事件字段参考 `Kaiheila 文档`

    .. Kaiheila 文档:
        https://developer.kaiheila.cn/doc/event/event-introduction
    """

    __event__ = ""
    channel_type: Literal["PERSON", "GROUP"]
    type_: int = Field(alias="type")
    """1:文字消息\n2:图片消息\n3:视频消息\n4:文件消息\n8:音频消息\n9:KMarkdown\n10:card消息\n255:系统消息\n其它的暂未开放"""
    target_id: str
    """
    发送目的\n
    频道消息类时, 代表的是频道 channel_id\n
    如果 channel_type 为 GROUP 组播且 type 为 255 系统消息时，则代表服务器 guild_id"""
    author_id: str = None
    content: str
    msg_id: str
    msg_timestamp: int
    nonce: str
    extra: Extra
    user_id: str

    post_type: str
    self_id: Optional[str] = None  # onebot兼容

    @override
    def get_event_description(self) -> str:
        return escape_tag(str(model_dump(self)))

    @override
    def get_plaintext(self) -> str:
        return self.content

    @override
    def get_user_id(self) -> str:
        return self.user_id

    @override
    def get_session_id(self) -> str:
        raise ValueError("Event has no message!")

    @override
    def is_tome(self) -> bool:
        return False


# Message Events
class MessageEvent(Event):
    """消息事件"""

    __event__ = "message"

    post_type: Literal["message"] = "message"
    message_type: str  # group private 其实是person
    sub_type: str
    event: EventMessage

    to_me: bool = False
    """
    :说明: 消息是否与机器人有关

    :类型: ``bool``
    """

    @override
    def get_event_name(self) -> str:
        sub_type = getattr(self, "sub_type", None)
        return f"{self.post_type}.{self.message_type}" + (
            f".{sub_type}" if sub_type else ""
        )

    @override
    def get_message(self) -> Message:
        return self.message

    @override
    def get_plaintext(self) -> str:
        return self.message.extract_plain_text()

    @override
    def is_tome(self) -> bool:
        return self.to_me

    @property
    def message(self) -> Message:
        return self.event.content


class PrivateMessageEvent(MessageEvent):
    """私聊消息"""

    __event__ = "message.private"
    message_type: Literal["private"]

    @override
    def get_session_id(self) -> str:
        return f"user_{self.user_id}"

    @override
    def get_event_description(self) -> str:
        return (
            f'Message {self.msg_id} from {self.user_id} "'
            + "".join(
                escape_tag(str(x)) if x.is_text() else f"<le>{escape_tag(str(x))}</le>"
                for x in self.message
            )
            + '"'
        )


class ChannelMessageEvent(MessageEvent):
    """公共频道消息"""

    __event__ = "message.group"
    message_type: Literal["group"]
    group_id: str

    @override
    def get_event_description(self) -> str:
        return (
            f'Message {self.msg_id} from {self.user_id}@[服务器:{self.extra.guild_id}][频道:{self.group_id}] "'
            + "".join(
                escape_tag(str(x)) if x.is_text() else f"<le>{escape_tag(str(x))}</le>"
                for x in self.event.content
            )
        )

    @override
    def get_session_id(self) -> str:
        return f"group_{self.group_id}_{self.user_id}"


# Notice Events
class NoticeEvent(Event):
    """通知事件"""

    __event__ = "notice"
    post_type: Literal["notice"]
    notice_type: str

    @override
    def get_event_name(self) -> str:
        sub_type = getattr(self, "sub_type", None)
        return f"{self.post_type}.{self.notice_type}" + (
            f".{sub_type}" if sub_type else ""
        )


# Channel Events
class ChannelNoticeEvent(NoticeEvent):
    """频道消息事件"""

    group_id: int

    @override
    def get_session_id(self) -> str:
        return f"group_{self.group_id}_{self.user_id}"


class ChannelAddReactionEvent(ChannelNoticeEvent):
    """频道内用户添加 reaction"""

    __event__ = "notice.added_reaction"
    notice_type: Literal["added_reaction"]

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Notice {self.msg_id} from {self.extra.body.user_id}@"
            f"[服务器:{self.target_id}]"
            f"[频道:{self.extra.body.channel_id}] "
            + f'add Emoji "{self.extra.body.emoji.name}" '
            + ""
            + f"to {self.extra.body.msg_id}"
            + '"'
        )


class ChannelDeletedReactionEvent(ChannelNoticeEvent):
    """频道内用户删除 reaction"""

    __event__ = "notice.deleted_reaction"
    notice_type: Literal["deleted_reaction"]

    @override
    def get_user_id(self) -> str:
        return str(self.user_id)

    @override
    def get_session_id(self) -> str:
        return f"group_{self.group_id}_{self.user_id}"

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Notice {self.msg_id} from {self.extra.body.user_id}@"
            f"[服务器:{self.target_id}]"
            f"[频道{self.extra.body.channel_id}] "
            + f'delete Emoji "{self.extra.body.emoji.name}" '
            + f"from {self.extra.body.msg_id}"
        )


class ChannelUpdatedMessageEvent(ChannelNoticeEvent):
    """频道消息更新"""

    __event__ = "notice.updated_message"
    notice_type: Literal["updated_message"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.user_id}@[服务器:{self.target_id}][频道{self.extra.body.channel_id}] "
            + f"update message {self.extra.body.msg_id} "
            + f'to "{self.extra.body.content}'
            + '"'
        )


class ChannelDeleteMessageEvent(ChannelNoticeEvent):
    """频道消息被删除"""

    __event__ = "notice.deleted_message"
    notice_type: Literal["deleted_message"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.user_id}@[服务器:{self.target_id}][频道{self.extra.body.channel_id}] "
            + f'delete message "{self.extra.body.msg_id}" '
        )


class ChannelAddedEvent(ChannelNoticeEvent):
    """新增频道"""

    __event__ = "notice.added_channel"
    notice_type: Literal["added_channel"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.user_id}@[服务器:{self.target_id}] "
            + f'新增频道 "[{self.extra.body.name}:{self.extra.body.id}]'
            + '"'
        )


class ChannelUpdatedEvent(ChannelNoticeEvent):
    """修改频道信息"""

    __event__ = "notice.updated_channel"
    notice_type: Literal["updated_channel"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.user_id}@[服务器:{self.target_id}] "
            + f'修改频道信息 "[{self.extra.body.name}:{self.extra.body.id}]'
            + '"'
        )


class ChannelDeleteEvent(ChannelNoticeEvent):
    """删除频道"""

    __event__ = "notice.deleted_channel"
    notice_type: Literal["deleted_channel"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.user_id}@[服务器:{self.target_id}] "
            + f'删除频道 "[{self.extra.body.id}]'
            + '"'
        )


class ChannelPinnedMessageEvent(ChannelNoticeEvent):
    """新增频道置顶消息"""

    __event__ = "notice.pinned_message"
    notice_type: Literal["pinned_message"]

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Notice {self.msg_id} from {self.extra.body.operator_id}@"
            f"[服务器:{self.target_id}]"
            f"[频道{self.extra.body.channel_id}] "
            + f'新增频道置顶消息 "{self.extra.body.msg_id}" '
        )


class ChannelUnpinnedMessageEvent(ChannelNoticeEvent):
    """取消频道置顶消息"""

    __event__ = "notice.unpinned_message"
    notice_type: Literal["unpinned_message"]

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Notice {self.msg_id} from {self.extra.body.operator_id}@"
            f"[服务器:{self.target_id}]"
            f"[频道{self.extra.body.channel_id}] "
            + f'取消频道置顶消息 "{self.extra.body.msg_id}" '
        )


# Private Events
class PrivateNoticeEvent(NoticeEvent):
    """私聊消息事件"""

    pass


class PrivateUpdateMessageEvent(PrivateNoticeEvent):
    """私聊消息更新"""

    __event__ = "notice.updated_private_message"
    notice_type: Literal["updated_private_message"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.extra.body.author_id} "
            + f"修改了消息 {self.extra.body.msg_id} "
            + f'to "{self.extra.body.content}'
            + '"'
        )


class PrivateDeleteMessageEvent(PrivateNoticeEvent):
    """私聊消息删除"""

    __event__ = "notice.deleted_private_message"
    notice_type: Literal["deleted_private_message"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.extra.body.author_id} "
            + f"删除了消息 {self.extra.body.msg_id} "
        )


class PrivateAddReactionEvent(PrivateNoticeEvent):
    """私聊内用户添加 reaction"""

    __event__ = "notice.private_added_reaction"
    notice_type: Literal["private_added_reaction"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.extra.body.user_id} "
            + f"add Emoji {self.extra.body.emoji.name} "
            + f'to "{self.extra.body.msg_id}'
            + '"'
        )


class PrivateDeleteReactionEvent(PrivateNoticeEvent):
    """私聊内用户取消 reaction"""

    __event__ = "notice.private_deleted_reaction"
    notice_type: Literal["private_deleted_reaction"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.extra.body.user_id} "
            + f"delete Emoji {self.extra.body.emoji.name} "
            + f'from "{self.extra.body.msg_id}'
            + '"'
        )


# Guild Events
class GuildNoticeEvent(NoticeEvent):
    """服务器相关事件"""

    group_id: int

    @override
    def get_session_id(self) -> str:
        return f"Guild_{self.group_id}_user_{self.user_id}"

    def get_guild_id(self):
        return self.target_id


# Guild Member Events
class GuildMemberNoticeEvent(GuildNoticeEvent):
    """服务器成员相关事件"""

    pass


class GuildMemberIncreaseNoticeEvent(GuildMemberNoticeEvent):
    """新成员加入服务器"""

    __event__ = "notice.joined_guild"
    notice_type: Literal["joined_guild"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'用户 "{self.extra.body.user_id}" 加入服务器'
        )


class GuildMemberDecreaseNoticeEvent(GuildMemberNoticeEvent):
    """服务器成员退出"""

    __event__ = "notice.exited_guild"
    notice_type: Literal["exited_guild"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'用户 "{self.extra.body.user_id}" 退出服务器'
        )


class GuildMemberUpdateNoticeEvent(GuildMemberNoticeEvent):
    """服务器成员信息更新(修改昵称)"""

    __event__ = "notice.updated_guild_member"
    notice_type: Literal["updated_guild_member"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'用户 "{self.extra.body.user_id}" 修改昵称为 "{self.extra.body.nickname}"'
        )


class GuildMemberOnlineNoticeEvent(GuildMemberNoticeEvent):
    """服务器成员上线"""

    __event__ = "notice.guild_member_online"
    notice_type: Literal["guild_member_online"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'成员 "{self.extra.body.user_id}" 上线'
        )


class GuildMemberOfflineNoticeEvent(GuildMemberNoticeEvent):
    """服务器成员下线"""

    __event__ = "notice.guild_member_offline"
    notice_type: Literal["guild_member_offline"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'成员 "{self.extra.body.user_id}" 离线'
        )


# Guild Role Events
class GuildRoleNoticeEvent(GuildNoticeEvent):
    """服务器角色相关事件"""

    pass


class GuildRoleAddNoticeEvent(GuildRoleNoticeEvent):
    """服务器角色增加"""

    __event__ = "notice.added_role"
    notice_type: Literal["added_role"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'增加角色 "{self.extra.body.role_id}:{self.extra.body.name}" '
        )


class GuildRoleDeleteNoticeEvent(GuildRoleNoticeEvent):
    """服务器角色增加"""

    __event__ = "notice.deleted_role"
    notice_type: Literal["deleted_role"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'删除角色 "{self.extra.body.role_id}:{self.extra.body.name}" '
        )


class GuildRoleUpdateNoticeEvent(GuildRoleNoticeEvent):
    """服务器角色增加"""

    __event__ = "notice.updated_role"
    notice_type: Literal["updated_role"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'更改角色 "{self.extra.body.role_id}:{self.extra.body.name}" '
        )


# Guild Events
class GuildUpdateNoticeEvent(GuildNoticeEvent):
    """服务器信息更新"""

    __event__ = "notice.updated_guild"
    notice_type: Literal["updated_guild"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'信息更新 "{self.extra.body.id}:{self.extra.body.name}" '
        )


class GuildDeleteNoticeEvent(GuildNoticeEvent):
    """服务器删除"""

    __event__ = "notice.deleted_guild"
    notice_type: Literal["deleted_guild"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}] "
            + f'删除 "{self.extra.body.id}:{self.extra.body.name}" '
        )


class GuildAddBlockListNoticeEvent(GuildNoticeEvent):
    """服务器封禁用户"""

    __event__ = "notice.added_block_list"
    notice_type: Literal["added_block_list"]

    @override
    def get_event_description(self) -> str:
        block_list = " ".join(self.extra.body.user_id)
        return (
            f"Notice {self.msg_id} from {self.extra.body.operator_id}@[服务器:{self.target_id}] "
            + f'封禁用户 "{self.extra.body.remark}:{block_list}" '
        )


class GuildDeleteBlockListNoticeEvent(GuildNoticeEvent):
    """服务器取消封禁用户"""

    __event__ = "notice.deleted_block_list"
    notice_type: Literal["deleted_block_list"]

    @override
    def get_event_description(self) -> str:
        block_list = " ".join(self.extra.body.user_id)
        return (
            f"Notice {self.msg_id} from {self.extra.body.operator_id}@[服务器:{self.target_id}] "
            + f'取消封禁用户 "{block_list}" '
        )


# User Events
class UserNoticeEvent(NoticeEvent):
    """用户相关事件列表"""

    group_id: int

    @override
    def get_session_id(self) -> str:
        return f"Guild_{self.group_id}_{self.user_id}"


class UserJoinAudioChannelNoticeEvent(UserNoticeEvent):
    """用户加入语音频道"""

    __event__ = "notice.joined_channel"
    notice_type: Literal["joined_channel"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}][频道{self.extra.body.channel_id}] "
            + f'用户 "{self.extra.body.user_id}" 加入语音频道 '
        )


class UserJoinAudioChannelEvent(UserNoticeEvent):
    """用户退出语音频道"""

    __event__ = "notice.exited_channel"
    notice_type: Literal["exited_channel"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id}@[服务器:{self.target_id}][频道{self.extra.body.channel_id}] "
            + f'用户 "{self.extra.body.user_id}" 退出语音频道 '
        )


class UserInfoUpdateNoticeEvent(UserNoticeEvent):
    """
    用户信息更新

    该事件与服务器无关, 遵循以下条件:
        - 仅当用户的 用户名 或 头像 变更时
        - 仅通知与该用户存在关联的用户或 Bot
            a. 存在聊天会话
            b. 双方好友关系
    """

    __event__ = "notice.user_updated"
    notice_type: Literal["user_updated"]

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id} "
            + f'用户变更信息 "{self.extra.body.username}:{self.extra.body.avatar}" '
        )


class SelfJoinGuildNoticeEvent(NoticeEvent):
    """
    自己新加入服务器

    当自己被邀请或主动加入新的服务器时, 产生该事件
    """

    __event__ = "notice.self_joined_guild"
    notice_type: Literal["self_joined_guild"]
    user_id: str
    group_id: int

    @override
    def get_user_id(self) -> str:
        return str(self.user_id)

    @override
    def get_session_id(self) -> str:
        return f"group_{self.group_id}_{self.user_id}"

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id} "
            + f'Bot 加入服务器 "{self.extra.body.guild_id}" '
        )


class SelfExitGuildNoticeEvent(NoticeEvent):
    """
    自己退出服务器

    当自己被踢出服务器或被拉黑或主动退出服务器时, 产生该事件
    """

    __event__ = "notice.self_exited_guild"
    notice_type: Literal["self_exited_guild"]
    user_id: str
    group_id: int

    @override
    def get_user_id(self) -> str:
        return str(self.user_id)

    @override
    def get_session_id(self) -> str:
        return f"group_{self.group_id}_{self.user_id}"

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id} "
            + f'Bot 退出服务器 "{self.extra.body.guild_id}" '
        )


class CartBtnClickNoticeEvent(NoticeEvent):
    """
    Card 消息中的 Button 点击事件
    """

    __event__ = "notice.message_btn_click"
    notice_type: Literal["message_btn_click"]
    user_id: str
    group_id: int

    @override
    def get_user_id(self) -> str:
        return str(self.user_id)

    @override
    def get_session_id(self) -> str:
        return f"group_{self.group_id}_{self.user_id}"

    @override
    def get_event_description(self) -> str:
        return (
            f"Notice {self.msg_id} from {self.author_id} "
            + f'用户 "{self.extra.body.user_id}@{self.extra.body.user_info.nickname}" 点击 Card 消息'
            + f' "{self.extra.body.msg_id}" 中的 Button '
        )


# Meta Events
class MetaEvent(OriginEvent):
    """元事件"""

    __event__ = "meta_event"
    post_type: Literal["meta_event"]
    meta_event_type: str

    @override
    def get_event_name(self) -> str:
        sub_type = getattr(self, "sub_type", None)
        return f"{self.post_type}.{self.meta_event_type}" + (
            f".{sub_type}" if sub_type else ""
        )

    @override
    def get_log_string(self) -> str:
        raise NoLogException


class LifecycleMetaEvent(MetaEvent):
    """生命周期元事件"""

    __event__ = "meta_event.lifecycle"
    meta_event_type: Literal["lifecycle"]
    sub_type: str

    session_id: str

    @override
    def get_session_id(self) -> str:
        return self.session_id


class HeartbeatMetaEvent(MetaEvent):
    """心跳元事件"""

    __event__ = "meta_event.heartbeat"
    meta_event_type: Literal["heartbeat"]


_t = StringTrie(separator=".")

# define `model` first to avoid globals changing while `for`
model = None
for model in globals().values():
    if not inspect.isclass(model) or not issubclass(model, OriginEvent):
        continue
    _t["." + model.__event__] = model


def get_event_model(event_name) -> List[Type[OriginEvent]]:
    """
    :说明:

      根据事件名获取对应 ``Event Model`` 及 ``FallBack Event Model`` 列表

    :返回:

      - ``List[Type[Event]]``
    """
    return [model.value for model in _t.prefixes("." + event_name)][::-1]


__all__ = [
    "EventTypes",
    "SignalTypes",
    "Role",
    "Channel",
    "User",
    "Guild",
    "Emoji",
    "Attachment",
    "Extra",
    "OriginEvent",
    "Kmarkdown",
    "EventMessage",
    "Event",
    "MessageEvent",
    "PrivateMessageEvent",
    "ChannelMessageEvent",
    "NoticeEvent",
    "ChannelNoticeEvent",
    "ChannelAddReactionEvent",
    "ChannelDeletedReactionEvent",
    "ChannelUpdatedMessageEvent",
    "ChannelDeleteMessageEvent",
    "ChannelAddedEvent",
    "ChannelUpdatedEvent",
    "ChannelDeleteEvent",
    "ChannelPinnedMessageEvent",
    "ChannelUnpinnedMessageEvent",
    "PrivateNoticeEvent",
    "PrivateUpdateMessageEvent",
    "PrivateDeleteMessageEvent",
    "PrivateAddReactionEvent",
    "PrivateDeleteReactionEvent",
    "GuildNoticeEvent",
    "GuildMemberNoticeEvent",
    "GuildMemberIncreaseNoticeEvent",
    "GuildMemberDecreaseNoticeEvent",
    "GuildMemberUpdateNoticeEvent",
    "GuildMemberOnlineNoticeEvent",
    "GuildMemberOfflineNoticeEvent",
    "GuildRoleNoticeEvent",
    "GuildRoleAddNoticeEvent",
    "GuildRoleDeleteNoticeEvent",
    "GuildRoleUpdateNoticeEvent",
    "GuildUpdateNoticeEvent",
    "GuildDeleteNoticeEvent",
    "GuildAddBlockListNoticeEvent",
    "GuildDeleteBlockListNoticeEvent",
    "UserNoticeEvent",
    "UserJoinAudioChannelNoticeEvent",
    "UserJoinAudioChannelEvent",
    "UserInfoUpdateNoticeEvent",
    "SelfJoinGuildNoticeEvent",
    "SelfExitGuildNoticeEvent",
    "CartBtnClickNoticeEvent",
    "MetaEvent",
    "LifecycleMetaEvent",
    "HeartbeatMetaEvent",
]



from typing import List, Optional, Tuple

from pydantic import Field, BaseModel
from nonebot.compat import PYDANTIC_V2, ConfigDict


class BotConfig(BaseModel):
    """
    Kaiheila Bot 配置类
    :配置项:
      - ``token``: Kaiheila 开发者中心获得
    """

    token: str

    if PYDANTIC_V2:
        model_config = ConfigDict(
            extra="ignore",
            populate_by_name=True,
        )
    else:

        class Config(ConfigDict):
            extra = "ignore"
            allow_population_by_field_name = True


class Config(BaseModel):
    """
    Kaiheila 配置类

    :配置项:

      - ``kaiheila_bots`` : Kaiheila 开发者中心获得
      - ``compress`` : 是否开启压缩, 默认为 False

    :示例:

    .. code-block:: default

        bots = [{"token": "token1"}, {"token": "token2"}]
    """

    kaiheila_bots: List["BotConfig"] = Field(default_factory=list)
    compress: Optional[bool] = Field(default=False)
    kaiheila_ignore_events: Tuple[str, ...] = Field(default_factory=tuple)
    kaiheila_ignore_other_bots: Optional[bool] = Field(default=True)

    if PYDANTIC_V2:
        model_config = ConfigDict(
            extra="allow",
            populate_by_name=True,
        )
    else:

        class Config(ConfigDict):
            extra = "allow"
            allow_population_by_field_name = True



import json
import warnings
from abc import ABC
from pathlib import Path
from dataclasses import dataclass
from collections.abc import Iterable
from typing_extensions import Self, override
from typing import TYPE_CHECKING, Any, Type, Union, Callable, Optional, TypedDict, cast

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment

from .utils import BytesReadable, escape_kmarkdown, unescape_kmarkdown
from .exception import (
    UnsupportedMessageType,
    KaiheilaAdapterException,
    UnsupportedMessageOperation,
)

if TYPE_CHECKING:
    from .bot import Bot


class MessageSegment(BaseMessageSegment["Message"], ABC):
    """
    开黑啦 协议 MessageSegment 适配。具体方法参考协议消息段类型或源码。

    https://developer.kaiheila.cn/doc/event/message
    """

    # 已知：
    # command/shell_command使用message_seg = Message[0]; str(message_seg) if message_seg.is_text()
    # startswith/endswith/keyword使用event.get_plaintext()
    # regex使用str(Message)

    @classmethod
    @override
    def get_message_class(cls) -> Type["Message"]:
        return Message

    def __str__(self) -> str:
        return "[未知类型消息]"

    @property
    def plain_text(self):
        return ""

    @classmethod
    def type_code(cls) -> int:
        """
        对应开黑啦协议里消息的type属性
        """
        return -1

    async def _serialize_for_send(self, bot: "Bot") -> Optional[dict]:
        return None

    def conduct(
        self, other: "Union[str, MessageSegment, Iterable[MessageSegment]]"
    ) -> "MessageSegment":
        """
        连接两个或多个 MessageSegment，必须为纯文本段或 KMarkdown 段
        """
        if isinstance(other, str):
            other = [Text.create(other)]
        elif isinstance(other, MessageSegment):
            other = [other]
        msg = Message([self, *other])
        msg.reduce()

        if len(msg) != 1:
            raise UnsupportedMessageOperation("必须为纯文本段或 KMarkdown 段")
        else:
            return msg[0]

    @override
    def is_text(self) -> bool:
        return False

    @staticmethod
    def text(text: str) -> "Text":
        """
        构造文本消息段

        @param text: 消息内容
        """
        return Text.create(text)

    @staticmethod
    def KMarkdown(content: str, raw_content: Optional[str] = None) -> "KMarkdown":
        """
        构造KMarkdown消息段

        @param content: KMarkdown消息内容（语法参考：https://developer.kookapp.cn/doc/kmarkdown）
        @param raw_content: （可选）消息段的纯文本内容
        """
        return KMarkdown.create(content, raw_content)

    @staticmethod
    def Card(content: Any) -> "Card":
        """
        构造卡片消息

        @param content: 卡片消息内容（语法参考：https://developer.kookapp.cn/doc/cardmessage）
        """
        return Card.create(content)

    @staticmethod
    def image(file_key: str) -> "Image":
        return Image.create(file_key)

    @staticmethod
    def local_image(file: Union[str, Path, BytesReadable, bytes]) -> "LocalImage":
        return LocalImage.create(file)

    @staticmethod
    def video(file_key: str, title: Optional[str] = None) -> "Video":
        return Video.create(file_key, title)

    @staticmethod
    def local_video(
        file: Union[str, Path, BytesReadable, bytes], title: Optional[str] = None
    ) -> "LocalVideo":
        return LocalVideo.create(file, title)

    @staticmethod
    def audio(
        file_key: str, title: Optional[str] = None, cover_file_key: Optional[str] = None
    ) -> "MessageSegment":
        return Audio.create(file_key, title, cover_file_key)

    @staticmethod
    def local_audio(
        file: Union[str, Path, BytesReadable, bytes],
        title: Optional[str] = None,
        cover: Union[None, str, Path, BytesReadable, bytes] = None,
    ) -> "LocalAudio":
        return LocalAudio.create(file, title, cover)

    @staticmethod
    def file(file_key: str, title: Optional[str] = None) -> "File":
        return File.create(file_key, title)

    @staticmethod
    def local_file(
        file: Union[str, Path, BytesReadable, bytes], title: Optional[str] = None
    ) -> "LocalFile":
        return LocalFile.create(file, title)

    @staticmethod
    def quote(msg_id: str) -> "Quote":
        return Quote.create(msg_id)

    @staticmethod
    def at(user_id: str) -> "Mention":
        warnings.warn(
            "用 MessageSegment.mention 代替",
            DeprecationWarning,
        )
        return MessageSegment.mention(user_id)

    @staticmethod
    def mention(user_id: str) -> "Mention":
        return Mention.create(user_id)

    @staticmethod
    def mention_role(role_id: str) -> "MentionRole":
        return MentionRole.create(role_id)

    @staticmethod
    def mention_all() -> "MentionAll":
        return MentionAll.create()

    @staticmethod
    def mention_here() -> "MentionHere":
        return MentionHere.create()


class ReceivableMessageSegment(MessageSegment):
    @classmethod
    def _deserialize(cls, raw_data: dict) -> Self:
        raise NotImplementedError()


class VirtualMessageSegment(MessageSegment):
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        return None


class Text(ReceivableMessageSegment):
    if TYPE_CHECKING:

        class _TextData(TypedDict):
            text: str

        data: _TextData

    @classmethod
    @override
    def type_code(cls) -> int:
        return 1

    @override
    def __str__(self) -> str:
        return self.data["text"]

    @property
    @override
    def plain_text(self):
        return self.data["text"]

    @plain_text.setter
    def plain_text(self, value: str):
        self.data["text"] = value

    @override
    def is_text(self) -> bool:
        return True

    @override
    async def _serialize_for_send(self, bot: "Bot") -> dict:
        return {"type": self.type_code(), "content": self.data["text"]}

    @classmethod
    @override
    def _deserialize(cls, raw_data: dict) -> Self:
        return cls.create(raw_data["content"])

    @classmethod
    def create(cls, text: str) -> "Text":
        return cls("text", {"text": text})


class KMarkdown(ReceivableMessageSegment):
    if TYPE_CHECKING:

        class _KMarkdownData(TypedDict):
            content: str
            raw_content: str

        data: _KMarkdownData

    @classmethod
    @override
    def type_code(cls) -> int:
        return 9

    @override
    def __str__(self) -> str:
        return self.data["content"]

    @property
    def plain_text(self):
        return self.data["raw_content"]

    @override
    def is_text(self) -> bool:
        return True

    @override
    async def _serialize_for_send(self, bot: "Bot") -> dict:
        return {"type": self.type_code(), "content": self.data["content"]}

    @classmethod
    @override
    def _deserialize(cls, raw_data: dict) -> Self:
        content = raw_data["content"]
        raw_content = raw_data["kmarkdown"]["raw_content"]

        # raw_content默认strip掉首尾空格，但是开黑啦本体的聊天界面中不会strip，所以这里还原了
        unescaped = unescape_kmarkdown(content)
        is_plain_text = unescaped.strip() == raw_content
        if is_plain_text:
            raw_content = unescaped

        return cls.create(content, raw_content)

    @classmethod
    def create(cls, content: str, raw_content: Optional[str] = None) -> "KMarkdown":
        if raw_content is None:
            raw_content = ""

        return cls("kmarkdown", {"content": content, "raw_content": raw_content})


class Card(ReceivableMessageSegment):
    if TYPE_CHECKING:

        class _CardData(TypedDict):
            content: str

        data: _CardData

    @classmethod
    @override
    def type_code(cls) -> int:
        return 10

    @override
    def __str__(self) -> str:
        return "[卡片消息]"

    @override
    async def _serialize_for_send(self, bot: "Bot") -> dict:
        return {"type": self.type_code(), "content": self.data["content"]}

    @classmethod
    @override
    def _deserialize(cls, raw_data: dict) -> Self:
        return cls.create(raw_data["content"])

    @classmethod
    def create(cls, content: Any) -> "Card":
        if not isinstance(content, str):
            content = json.dumps(content)

        return cls("card", {"content": content})


class Media(ReceivableMessageSegment):
    if TYPE_CHECKING:

        class _MediaData(TypedDict):
            file_key: str

        data: _MediaData

    @override
    async def _serialize_for_send(self, bot: "Bot") -> dict:
        return {"type": self.type_code(), "content": self.data["file_key"]}


class Image(Media):
    @classmethod
    @override
    def type_code(cls) -> int:
        return 2

    @override
    def __str__(self) -> str:
        return "[图片]"

    @classmethod
    @override
    def _deserialize(cls, raw_data: dict) -> Self:
        return cls.create(raw_data["attachments"]["url"])

    @classmethod
    def create(cls, file_key: str) -> "Image":
        return cls("image", {"file_key": file_key})


class Video(Media):
    if TYPE_CHECKING:

        class _VideoData(Media._MediaData):
            title: Optional[str] = None

        data: _VideoData

    @classmethod
    @override
    def type_code(cls) -> int:
        return 3

    @override
    def __str__(self) -> str:
        return "[视频]"

    @classmethod
    @override
    def _deserialize(cls, raw_data: dict) -> Self:
        return cls.create(
            raw_data["attachments"]["url"], raw_data["attachments"]["name"]
        )

    @classmethod
    def create(cls, file_key: str, title: Optional[str] = None) -> "Video":
        return cls("video", {"file_key": file_key, "title": title})


class Audio(Media):
    if TYPE_CHECKING:

        class _AudioData(Media._MediaData):
            title: Optional[str] = None
            cover_file_key: Optional[str] = None

        data: _AudioData

    @classmethod
    @override
    def type_code(cls) -> int:
        return 8

    @override
    def __str__(self) -> str:
        return "[音频]"

    async def _serialize_for_send(self, bot: "Bot") -> dict:
        # 转化为卡片消息发送
        return await _convert_to_card_message(Message(self))._serialize_for_send(bot)

    @classmethod
    @override
    def _deserialize(cls, raw_data: dict) -> Self:
        return cls.create(
            raw_data["attachments"]["url"], raw_data["attachments"]["name"]
        )

    @classmethod
    def create(
        cls,
        file_key: str,
        title: Optional[str] = None,
        cover_file_key: Optional[str] = None,
    ) -> "Audio":
        return cls(
            "audio",
            {"file_key": file_key, "title": title, "cover_file_key": cover_file_key},
        )


class File(Media):
    if TYPE_CHECKING:

        class _FileData(Media._MediaData):
            title: Optional[str] = None

        data: _FileData

    @classmethod
    @override
    def type_code(cls) -> int:
        return 4

    @override
    def __str__(self) -> str:
        return "[文件]"

    async def _serialize_for_send(self, bot: "Bot") -> dict:
        # 转化为卡片消息发送
        return await _convert_to_card_message(Message(self))._serialize_for_send(bot)

    @classmethod
    @override
    def _deserialize(cls, raw_data: dict) -> Self:
        return cls.create(
            raw_data["attachments"]["url"], raw_data["attachments"]["name"]
        )

    @classmethod
    def create(cls, file_key: str, title: Optional[str] = None) -> "File":
        return cls("file", {"file_key": file_key, "title": title})


class LocalMedia(VirtualMessageSegment):
    if TYPE_CHECKING:

        class _LocalMediaData(TypedDict):
            content: Optional[bytes] = None
            title: Optional[str] = None
            file: Optional[Path] = None

        data: _LocalMediaData

    async def _upload(self, bot: "Bot") -> str:
        if self.data["file"] is None and self.data["content"] is None:
            raise KaiheilaAdapterException("file_path 与 content 均为 None")

        file_key = await bot.upload_file(
            self.data["content"] or self.data["file"], self.data["title"]
        )
        return file_key

    @classmethod
    def _handle_file(
        cls, file: Union[str, Path, BytesReadable, bytes], title: Optional[str] = None
    ) -> "LocalMedia._LocalMediaData":
        data = {"title": title, "content": None, "file": None}

        if isinstance(file, BytesReadable):
            data["content"] = file.read()
        elif isinstance(file, bytes):
            data["content"] = file
        else:
            data["file"] = Path(file)
            if title is None:
                data["title"] = data["file"].name

        return data


class LocalImage(LocalMedia):
    @classmethod
    @override
    def type_code(cls) -> int:
        return Image.type_code()

    @override
    def __str__(self) -> str:
        return "[本地图片]"

    @override
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        file_key = await self._upload(bot)
        return Image.create(file_key)

    @classmethod
    def create(cls, file: Union[str, Path, BytesReadable, bytes]) -> "LocalImage":
        data = cls._handle_file(file)
        return cls("local_image", data)


class LocalVideo(LocalMedia):
    @classmethod
    @override
    def type_code(cls) -> int:
        return Video.type_code()

    @override
    def __str__(self) -> str:
        return "[本地视频]"

    @override
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        file_key = await self._upload(bot)
        return Video.create(file_key, self.data["title"])

    @classmethod
    def create(
        cls, file: Union[str, Path, BytesReadable, bytes], title: Optional[str] = None
    ) -> "LocalVideo":
        data = cls._handle_file(file, title)
        return cls("local_video", data)


class LocalFile(LocalMedia):
    @classmethod
    @override
    def type_code(cls) -> int:
        return File.type_code()

    @override
    def __str__(self) -> str:
        return "[本地文件]"

    @override
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        file_key = await self._upload(bot)
        return File.create(file_key, self.data["title"])

    @classmethod
    def create(
        cls,
        file: Union[str, Path, BytesReadable, bytes],
        filename: Optional[str] = None,
    ) -> "LocalFile":
        data = cls._handle_file(file, filename)
        return cls("local_file", data)


class LocalAudio(LocalMedia):
    if TYPE_CHECKING:

        class _LocalAudioData(LocalMedia._LocalMediaData):
            cover_content: Optional[bytes] = None
            cover_file: Optional[Path] = None

        data: _LocalAudioData

    @classmethod
    @override
    def type_code(cls) -> int:
        return Audio.type_code()

    @override
    def __str__(self) -> str:
        return "[本地音频]"

    @override
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        file_key = await self._upload(bot)

        if self.data["cover_content"] or self.data["cover_file"]:
            cover_file_key = await bot.upload_file(
                self.data["cover_content"] or self.data["cover_file"]
            )
        else:
            cover_file_key = None

        return Audio.create(file_key, self.data["title"], cover_file_key)

    @classmethod
    def create(
        cls,
        file: Union[str, Path, BytesReadable, bytes],
        filename: Optional[str] = None,
        cover: Union[None, str, Path, BytesReadable, bytes] = None,
    ) -> "LocalAudio":
        data = cls._handle_file(file, filename)

        if cover is not None:
            cover_data = cls._handle_file(cover)
            cover_data = {
                "cover_content": cover_data["content"],
                "cover_file": cover_data["file"],
            }
        else:
            cover_data = {"cover_content": None, "cover_file": None}

        data = {**data, **cover_data}
        return cls("local_audio", data)


class Quote(VirtualMessageSegment):
    if TYPE_CHECKING:

        class _QuoteData(TypedDict):
            msg_id: str

        data: _QuoteData

    @override
    def __str__(self) -> str:
        return "[引用消息]"

    @classmethod
    def create(cls, msg_id: str) -> "Quote":
        return cls("quote", {"msg_id": msg_id, "for_send": True})


class Mention(VirtualMessageSegment):
    if TYPE_CHECKING:

        class _MentionData(TypedDict):
            user_id: str
            username: Optional[str] = None

        data: _MentionData

    @override
    def __str__(self) -> str:
        if self.data["username"]:
            return f"@{self.data['username']}"
        else:
            return f"@用户{self.data['user_id']}"

    @override
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        return KMarkdown.create(f"(met){self.data['user_id']}(met)", str(self))

    @classmethod
    def create(cls, user_id: str, username: Optional[str] = None) -> "Mention":
        return cls(
            "mention", {"user_id": user_id, "username": username, "for_send": True}
        )


class MentionRole(VirtualMessageSegment):
    if TYPE_CHECKING:

        class _MentionData(TypedDict):
            role_id: str
            name: Optional[str] = None

        data: _MentionData

    @override
    def __str__(self) -> str:
        if self.data["name"]:
            return f"@{self.data['name']}"
        else:
            return f"@角色{self.data['role_id']}"

    @override
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        return KMarkdown.create(f"(rol){self.data['role_id']}(rol)", str(self))

    @classmethod
    def create(cls, role_id: str, name: Optional[str] = None) -> "MentionRole":
        return cls("mention_role", {"role_id": role_id, "name": name, "for_send": True})


class MentionAll(VirtualMessageSegment):
    @override
    def __str__(self) -> str:
        return "@全体成员"

    @override
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        return KMarkdown.create("(met)all(met)", str(self))

    @classmethod
    def create(cls) -> "MentionAll":
        return cls("mention_all", {"for_send": True})


class MentionHere(VirtualMessageSegment):
    @override
    def __str__(self) -> str:
        return "@在线成员"

    @override
    async def _actual_seg(self, bot: "Bot") -> Optional[MessageSegment]:
        return KMarkdown.create("(met)here(met)", str(self))

    @classmethod
    def create(cls) -> "MentionHere":
        return cls("mention_here", {"for_send": True})


class Message(BaseMessage[MessageSegment]):
    """
    开黑啦 v3 协议 Message 适配。
    """

    @classmethod
    @override
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @staticmethod
    @override
    def _construct(msg: str) -> "Iterable[MessageSegment]":
        yield Text.create(msg)

    @override
    def extract_plain_text(self) -> str:
        return "".join(seg.plain_text for seg in self)

    def reduce(self) -> None:
        """合并消息内连续的纯文本段和 KMarkdown 段。"""
        index = 1
        while index < len(self):
            prev = self[index - 1]
            cur = self[index]

            if isinstance(prev, Text) and isinstance(cur, Text):
                self[index - 1] = Text.create(prev.data["text"] + cur.data["text"])
                del self[index]
            elif isinstance(prev, KMarkdown) and isinstance(cur, KMarkdown):
                self[index - 1] = KMarkdown.create(
                    prev.data["content"] + cur.data["content"],
                    prev.data["raw_content"] + cur.data["raw_content"],
                )
                del self[index]
            elif isinstance(prev, KMarkdown) and isinstance(cur, Text):
                self[index - 1] = KMarkdown.create(
                    prev.data["content"] + escape_kmarkdown(cur.data["text"]),
                    prev.data["raw_content"] + cur.data["text"],
                )
                del self[index]
            elif isinstance(prev, Text) and isinstance(cur, KMarkdown):
                self[index - 1] = MessageSegment.KMarkdown(
                    escape_kmarkdown(prev.data["text"]) + cur.data["content"],
                    prev.data["text"] + cur.data["raw_content"],
                )
                del self[index]
            else:
                index += 1


def _convert_to_card_message(msg: Message) -> MessageSegment:
    cards = []
    modules = []

    for seg in msg:
        if isinstance(seg, Card):
            if len(modules) != 0:
                cards.append(
                    {"type": "card", "theme": "none", "size": "lg", "modules": modules}
                )
                modules = []
            cards.extend(json.loads(seg.data["content"]))
        elif isinstance(seg, Text):
            modules.append(
                {
                    "type": "section",
                    "text": {"type": "plain-text", "content": seg.data["text"]},
                }
            )
        elif isinstance(seg, KMarkdown):
            modules.append(
                {
                    "type": "section",
                    "text": {"type": "kmarkdown", "content": seg.data["content"]},
                }
            )
        elif isinstance(seg, Image):
            modules.append(
                {
                    "type": "container",
                    "elements": [{"type": "image", "src": seg.data["file_key"]}],
                }
            )
        elif isinstance(seg, (Audio, Video, File)):
            mod = {
                "type": seg.type,
                "src": seg.data["file_key"],
            }
            if seg.data.get("title") is not None:
                mod["title"] = seg.data["title"]
            if seg.data.get("cover_file_key") is not None:
                mod["cover"] = seg.data["cover_file_key"]
            modules.append(mod)
        else:
            raise UnsupportedMessageType(seg.type)

    if len(modules) != 0:
        cards.append(
            {"type": "card", "theme": "none", "size": "lg", "modules": modules}
        )

    return Card.create(cards)


@dataclass
class MessageSerializer:
    """
    开黑啦 协议 Message 序列化器。
    """

    message: Message

    async def serialize(self, bot: "Bot") -> dict:
        serialized_data = {}

        # quote
        quote = self.message.get("quote")
        if len(quote) > 0:
            self.message = self.message.exclude("quote")
            serialized_data["quote"] = cast(Quote, quote[-1]).data["msg_id"]

        # 将虚拟消息段转换为真实消息段
        new_message = Message()
        for seg in self.message:
            if isinstance(seg, VirtualMessageSegment):
                actual = await seg._actual_seg(bot)
                if actual is not None:
                    new_message.append(actual)
            else:
                new_message.append(seg)
        self.message = new_message

        # 大于一段时，先尝试合并text与kmarkdown
        if len(self.message) != 1:
            self.message = self.message.copy()
            self.message.reduce()

        # 文字与媒体混发时，转化为卡片消息发送
        if len(self.message) != 1:
            card_msg = Message(_convert_to_card_message(self.message))
            serialized_data = {
                **serialized_data,
                **(await MessageSerializer(card_msg).serialize(bot)),
            }
        else:
            serialized_data = {
                **serialized_data,
                **(await self.message[0]._serialize_for_send(bot)),
            }
        return serialized_data


_msg_type_map = {
    Text: Text.type_code(),
    Image: Image.type_code(),
    Video: Video.type_code(),
    File: File.type_code(),
    Audio: Audio.type_code(),
    KMarkdown: KMarkdown.type_code(),
    Card: Card.type_code(),
}

_rev_msg_type_map = {}
for msg_type, code in _msg_type_map.items():
    _rev_msg_type_map[code] = msg_type


@dataclass
class MessageDeserializer:
    """
    开黑啦 协议 Message 反序列化器。
    """

    type_code: int
    data: dict

    def __post_init__(self):
        self.type = _rev_msg_type_map.get(self.type_code, "")

    def kmd_with_raw_mention(self, content: str) -> str:
        # 将content中的(met)xxx(met)替换为@xxx
        # 在此基础上再去判断是否为纯文本消息

        content_with_raw_mention = content
        for mention in self.data["kmarkdown"]["mention_part"]:
            content_with_raw_mention = content_with_raw_mention.replace(
                f"(met){mention['id']}(met)", f"@{mention['username']}"
            )

        for mention in self.data["kmarkdown"]["mention_role_part"]:
            content_with_raw_mention = content_with_raw_mention.replace(
                f"(rol){mention['role_id']}(rol)", f"@{mention['name']}"
            )

        if self.data.get("mention_all"):
            content_with_raw_mention = content_with_raw_mention.replace(
                "(met)all(met)", "@全体成员"
            )

        if self.data.get("mention_here"):
            content_with_raw_mention = content_with_raw_mention.replace(
                "(met)here(met)", "@在线成员"
            )

        return content_with_raw_mention

    def is_kmd_plain_text(self, content: str, raw_content: str) -> bool:
        unescaped = unescape_kmarkdown(content)
        return unescaped.strip() == raw_content  # raw_content默认strip掉首尾空格

    def split_text(
        self,
        message: Message,
        find_text: str,
        replace_with: Callable[[], MessageSegment],
    ) -> Message:
        new_message = Message()
        for seg in message:
            if isinstance(seg, Text):
                text = seg.data["text"]
                rest_part = text.split(find_text)

                new_message.append(rest_part[0])
                for i in range(1, len(rest_part)):
                    new_message.append(replace_with())
                    new_message.append(Text.create(rest_part[i]))
            else:
                new_message.append(seg)
        return new_message

    def convert_mention_seg(self, message: Message) -> Message:
        for mention in self.data["kmarkdown"]["mention_part"]:
            message = self.split_text(
                message,
                f"(met){mention['id']}(met)",
                lambda: Mention.create(mention["id"], mention["username"]),
            )

        for mention in self.data["kmarkdown"]["mention_role_part"]:
            message = self.split_text(
                message,
                f"(met){mention['role_id']}(met)",
                lambda: MentionRole.create(mention["role_id"], mention["name"]),
            )

        if self.data.get("mention_all"):
            message = self.split_text(
                message, "(met)all(met)", lambda: MentionAll.create()
            )

        if self.data.get("mention_here"):
            message = self.split_text(
                message, "(met)here(met)", lambda: MentionHere.create()
            )

        # 去除首尾空文本消息段
        if (
            len(message) > 1
            and isinstance(message[0], Text)
            and len(message[0].plain_text) == 0
        ):
            message.pop(0)
        if (
            len(message) > 1
            and isinstance(message[-1], Text)
            and len(message[-1].plain_text) == 0
        ):
            message.pop()
        return message

    def deserialize(self) -> Message:
        if self.type == KMarkdown:
            content: str = self.data["content"]
            raw_content: str = self.data["kmarkdown"]["raw_content"]

            content_with_raw_mention = self.kmd_with_raw_mention(content)

            # 如果KMarkdown消息是纯文本（除了mention的部分以外），直接构造纯文本消息
            # 目的是让on_command等依赖__str__的规则能够在消息存在转义字符时正常工作
            if self.is_kmd_plain_text(content_with_raw_mention, raw_content):
                # 从反转义的kmarkdown构造，并将里面的mention部分换为对应消息段
                msg = Message(Text.create(unescape_kmarkdown(content)))
                msg = self.convert_mention_seg(msg)
            else:
                msg = Message(KMarkdown.create(content, raw_content))
        else:
            msg = Message(self.type._deserialize(self.data))

        return msg



import json
from typing import Optional

from nonebot.drivers import Response
from nonebot.exception import AdapterException
from nonebot.exception import ActionFailed as BaseActionFailed
from nonebot.exception import NetworkError as BaseNetworkError
from nonebot.exception import NoLogException as BaseNoLogException
from nonebot.exception import ApiNotAvailable as BaseApiNotAvailable


class KaiheilaAdapterException(AdapterException):
    def __init__(self, *args: object):
        super().__init__("Kaiheila", *args)


class NoLogException(BaseNoLogException, KaiheilaAdapterException):
    pass


class ActionFailed(BaseActionFailed, KaiheilaAdapterException):
    """
    :说明:

        API 请求返回错误信息。

    :参数:

        * ``response: Response``: 响应体
    """

    def __init__(self, response: Response):
        self.status_code: int = response.status_code
        self.code: Optional[int] = None
        self.message: Optional[str] = None
        self.data: Optional[dict] = None
        if response.content:
            body = json.loads(response.content)
            self._prepare_body(body)

    def __repr__(self):
        return (
            f"<ActionFailed: {self.status_code}, code={self.code}, "
            f"message={self.message}, data={self.data}>"
        )

    def __str__(self):
        return self.__repr__()

    def _prepare_body(self, body: dict):
        self.code = body.get("code", None)
        self.message = body.get("message", None)
        self.data = body.get("data", None)


class UnauthorizedException(ActionFailed):
    pass


class RateLimitException(ActionFailed):
    pass


class NetworkError(BaseNetworkError, KaiheilaAdapterException):
    """
    :说明:

      网络错误。

    :参数:

      * ``msg: Optional[int]``: 错误信息
    """

    def __init__(self, msg: Optional[str] = None):
        super().__init__()
        self.msg = msg

    def __repr__(self):
        return f"<NetWorkError message={self.msg}>"

    def __str__(self):
        return self.__repr__()


class ApiNotAvailable(BaseApiNotAvailable, KaiheilaAdapterException):
    pass


class UnsupportedMessageType(KaiheilaAdapterException):
    """
    :说明:

      在发送不支持的消息类型时抛出。
    """

    def __init__(self, message: str = ""):
        super().__init__()
        self.message = message

    def __repr__(self) -> str:
        return self.message


class UnsupportedMessageOperation(KaiheilaAdapterException):
    """
    :说明:

      在调用不支持的 Message 或 MessageSegment 操作时抛出。
    """

    def __init__(self, message: str = ""):
        super().__init__()
        self.message = message

    def __repr__(self) -> str:
        return self.message


class ReconnectError(KaiheilaAdapterException):
    """
    :说明:

      服务端通知客户端, 代表该连接已失效, 请重新连接。客户端收到后应该主动断开当前连接。
    """

    pass


class TokenError(KaiheilaAdapterException):
    """
    :说明:

      服务端通知客户端, 代表该连接已失效, 请重新连接。客户端收到后应该主动断开当前连接。
    """

    def __init__(self, msg: Optional[str] = None):
        super().__init__()
        self.msg = msg

    def __repr__(self):
        return f"<TokenError message={self.msg}>"

    def __str__(self):
        return self.__repr__()



from .model import (
    URL,
    Role,
    User,
    Guild,
    Channel,
    UserChat,
    GuildEmoji,
    RolesReturn,
    GuildsReturn,
    ReactionUser,
    DirectMessage,
    InvitesReturn,
    ChannelMessage,
    ChannelsReturn,
    ChannelRoleInfo,
    GuildRoleReturn,
    UserChatsReturn,
    BlackListsReturn,
    GuildBoostReturn,
    GuildUsersReturn,
    ChannelRoleReturn,
    GuildEmojisReturn,
    IntimacyIndexReturn,
    MessageCreateReturn,
    DirectMessagesReturn,
    ChannelMessagesReturn,
    ChannelRoleSyncResult,
    GetUserJoinedChannelReturn,
)

class ApiClient:
    async def asset_create(self, *, file) -> URL: ...
    async def blacklist_create(
        self,
        *,
        guild_id: str,
        target_id: str,
        remark: str | None = ...,
        del_msg_days: str | None = ...,
    ) -> None: ...
    async def blacklist_delete(self, *, guild_id: str, target_id: str) -> None: ...
    async def blacklist_list(self, *, guild_id: str) -> BlackListsReturn: ...
    async def channelUser_getJoinedChannel(
        self,
        *,
        guild_id: str,
        user_id: str,
        page: int | None = ...,
        page_size: int | None = ...,
    ) -> GetUserJoinedChannelReturn: ...
    async def channelRole_create(
        self, *, channel_id: str, type: str | None = ..., value: str | None = ...
    ) -> ChannelRoleReturn: ...
    async def channelRole_delete(
        self,
        *,
        channel_id: str,
        type: str | None = ...,
        value: str | None = ...,
    ) -> None: ...
    async def channelRole_index(self, *, channel_id: str) -> ChannelRoleInfo: ...
    async def channelRole_update(
        self,
        *,
        channel_id: str,
        type: str | None = ...,
        value: str | None = ...,
        allow: int | None = ...,
        deny: int | None = ...,
    ) -> ChannelRoleReturn: ...
    async def channelRole_sync(
        self,
        *,
        channel_id: str,
    ) -> ChannelRoleSyncResult: ...
    async def channel_create(
        self,
        *,
        guild_id: str,
        name: str,
        parent_id: str | None = ...,
        type: int | None = ...,
        limit_amount: int | None = ...,
        voice_quality: str | None = ...,
        is_category: int | None = ...,
    ) -> Channel: ...
    async def channel_delete(self, *, channel_id: str) -> None: ...
    async def channel_update(
        self,
        *,
        channel_id: str,
        name: str | None = ...,
        topic: str | None = ...,
        slow_mode: int | None = ...,
    ) -> Channel: ...
    async def channel_list(
        self,
        *,
        guild_id: str,
        type: int | None = ...,
        page: int | None = ...,
        page_size: int | None = ...,
    ) -> ChannelsReturn: ...
    async def channel_moveUser(
        self,
        *,
        target_id: str,
        user_ids: list[int],
    ) -> None: ...
    async def channel_userList(self, *, channel_id: str) -> list[User]: ...
    async def channel_view(self, *, target_id: str) -> Channel: ...
    async def directMessage_addReaction(self, *, msg_id: str, emoji: str) -> None: ...
    async def directMessage_create(
        self,
        *,
        content: str,
        type: int | None = ...,
        target_id: str | None = ...,
        chat_code: str | None = ...,
        quote: str | None = ...,
        nonce: str | None = ...,
    ) -> MessageCreateReturn: ...
    async def directMessage_delete(self, *, msg_id: str) -> None: ...
    async def directMessage_deleteReaction(
        self, *, msg_id: str, emoji: str, user_id: str | None = ...
    ) -> None: ...
    async def directMessage_list(
        self,
        *,
        chat_code: str | None = ...,
        target_id: str | None = ...,
        msg_id: str | None = ...,
        flag: str | None = ...,
        page: int | None = ...,
        page_size: int | None = ...,
    ) -> DirectMessagesReturn: ...
    async def directMessage_reactionList(
        self, *, msg_id: str, emoji: str
    ) -> list[ReactionUser]: ...
    async def directMessage_update(
        self, *, content: str, msg_id: str | None = ..., quote: str | None = ...
    ) -> None: ...
    async def directMessage_view(
        self, *, chat_code: str, msg_id: str
    ) -> DirectMessage: ...
    async def gateway_index(self, *, compress: int | None = ...) -> URL: ...
    async def guildBoost_history(
        self, *, guild_id: str, start_time: int | None = ..., end_time: int | None = ...
    ) -> GuildBoostReturn: ...
    async def guildEmoji_create(
        self, *, guild_id: str, emoji: bytes | None = ..., name: str | None = ...
    ) -> GuildEmoji: ...
    async def guildEmoji_delete(self, *, id: str) -> None: ...
    async def guildEmoji_list(
        self, *, guild_id: str, page: int | None = ..., page_size: int | None = ...
    ) -> GuildEmojisReturn: ...
    async def guildEmoji_update(self, *, id: str, name: str) -> None: ...
    async def guildMute_create(
        self, *, guild_id: str = ..., target_id: str = ..., type: int = ...
    ) -> None: ...
    async def guildMute_delete(
        self, *, guild_id: str = ..., target_id: str = ..., type: int = ...
    ) -> None: ...
    async def guildMute_list(
        self, *, guild_id: str, return_type: str | None = ...
    ) -> None: ...
    async def guildRole_create(
        self, *, guild_id: str, name: str | None = ...
    ) -> Role: ...
    async def guildRole_delete(self, *, guild_id: str, role_id: int) -> None: ...
    async def guildRole_grant(
        self, *, guild_id: str, user_id: str, role_id: int
    ) -> GuildRoleReturn: ...
    async def guildRole_list(
        self, *, guild_id: str, page: int | None = ..., page_size: int | None = ...
    ) -> RolesReturn: ...
    async def guildRole_revoke(
        self, *, guild_id: str, user_id: str, role_id: int
    ) -> GuildRoleReturn: ...
    async def guildRole_update(
        self,
        *,
        guild_id: str,
        role_id: int,
        name: str | None = ...,
        color: int | None = ...,
        hoist: int | None = ...,
        mentionable: int | None = ...,
        permissions: int | None = ...,
    ) -> Role: ...
    async def guild_kickout(self, *, guild_id: str, target_id: str) -> None: ...
    async def guild_leave(self, *, guild_id: str) -> None: ...
    async def guild_list(
        self,
        *,
        page: int | None = ...,
        page_size: int | None = ...,
        sort: str | None = ...,
    ) -> GuildsReturn: ...
    async def guild_nickname(
        self,
        *,
        guild_id: str = ...,
        nickname: str | None = ...,
        user_id: str | None = ...,
    ) -> None: ...
    async def guild_userList(
        self,
        *,
        guild_id: str,
        channel_id: str | None = ...,
        search: str | None = ...,
        role_id: int | None = ...,
        mobile_verified: int | None = ...,
        active_time: int | None = ...,
        joined_at: int | None = ...,
        page: int | None = ...,
        page_size: int | None = ...,
        filter_user_id: str | None = ...,
    ) -> GuildUsersReturn: ...
    async def guild_view(self, *, guild_id: str) -> Guild: ...
    async def intimacy_index(self, *, user_id: str) -> IntimacyIndexReturn: ...
    async def intimacy_update(
        self,
        *,
        user_id: str,
        score: int | None = ...,
        social_info: str | None = ...,
        img_id: str | None = ...,
    ) -> None: ...
    async def invite_create(
        self,
        *,
        guild_id: str | None = ...,
        channel_id: str | None = ...,
        duration: int | None = ...,
        setting_times: int | None = ...,
    ) -> URL: ...
    async def invite_delete(
        self, *, url_code: str, guild_id: str | None = ..., channel_id: str | None = ...
    ) -> None: ...
    async def invite_list(
        self,
        *,
        guild_id: str | None = ...,
        channel_id: str | None = ...,
        page: int | None = ...,
        page_size: int | None = ...,
    ) -> InvitesReturn: ...
    async def message_addReaction(self, *, msg_id: str, emoji: str) -> None: ...
    async def message_create(
        self,
        *,
        content: str,
        target_id: str,
        type: int | None = ...,
        quote: str | None = ...,
        nonce: str | None = ...,
        temp_target_id: str | None = ...,
    ) -> MessageCreateReturn: ...
    async def message_delete(self, *, msg_id: str) -> None: ...
    async def message_deleteReaction(
        self, *, msg_id: str, emoji: str, user_id: str | None = ...
    ) -> None: ...
    async def message_list(
        self,
        *,
        target_id: str,
        msg_id: str | None = ...,
        pin: int | None = ...,
        flag: str | None = ...,
        page_size: int | None = ...,
    ) -> ChannelMessagesReturn: ...
    async def message_reactionList(
        self, *, msg_id: str, emoji: str
    ) -> list[ReactionUser]: ...
    async def message_update(
        self,
        *,
        msg_id: str,
        content: str,
        quote: str | None = ...,
        temp_target_id: str | None = ...,
    ) -> None: ...
    async def message_view(self, *, msg_id: str) -> ChannelMessage: ...
    async def userChat_create(self, *, target_id: str) -> UserChat: ...
    async def userChat_delete(self, *, chat_code: str) -> None: ...
    async def userChat_list(
        self, *, page: int | None = ..., page_size: int | None = ...
    ) -> UserChatsReturn: ...
    async def userChat_view(self, *, chat_code: str) -> UserChat: ...
    async def user_me(self) -> User: ...
    async def user_offline(self) -> None: ...
    async def user_view(self, *, user_id: str, guild_id: str | None = ...) -> User: ...



import re
import json
import zlib
import asyncio
import inspect
from typing_extensions import override
from typing import Any, Dict, List, Tuple, Type, Union, Mapping, Callable, Optional

from pygtrie import StringTrie
from nonebot.utils import escape_tag
from nonebot.internal.driver import Response
from nonebot.compat import model_dump, type_validate_python
from nonebot.drivers import (
    URL,
    Driver,
    Request,
    WebSocket,
    ForwardDriver,
    HTTPClientMixin,
    WebSocketClientMixin,
)

from nonebot import get_plugin_config
from nonebot.adapters import Adapter as BaseAdapter

from . import event
from .bot import Bot
from .api.model import User
from .config import BotConfig
from .config import Config as KaiheilaConfig
from .message import Message, MessageSegment
from .api.handle import get_api_method, get_api_restype
from .utils import ResultStore, log, _handle_api_result
from .event import (
    Event,
    EventTypes,
    OriginEvent,
    SignalTypes,
    HeartbeatMetaEvent,
    LifecycleMetaEvent,
)
from .exception import (
    TokenError,
    ActionFailed,
    NetworkError,
    ReconnectError,
    ApiNotAvailable,
    RateLimitException,
    UnauthorizedException,
    KaiheilaAdapterException,
)

RECONNECT_INTERVAL = 3.0


class Adapter(BaseAdapter):
    # init all event models
    event_models: StringTrie = StringTrie(separator=".")
    for model_name in dir(event):
        model = getattr(event, model_name)
        if not inspect.isclass(model) or not issubclass(model, OriginEvent):
            continue
        event_models["." + model.__event__] = model

    @override
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)
        self.kaiheila_config: KaiheilaConfig = get_plugin_config(KaiheilaConfig)
        self.api_root = "https://www.kaiheila.cn/api/v3/"
        self.connections: Dict[str, WebSocket] = {}
        self.tasks: List[asyncio.Task] = []
        self.setup()

    # OK
    @classmethod
    @override
    def get_name(cls) -> str:
        return "Kaiheila"

    # OK
    def setup(self) -> None:
        if not isinstance(self.driver, HTTPClientMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} does not support "
                "http client requests! "
                f"{self.get_name()} Adapter need a HTTPClient Driver to work."
            )
        if not isinstance(self.driver, WebSocketClientMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} does not support "
                "websocket client! "
                f"{self.get_name()} Adapter need a WebSocketClient Driver to work."
            )
        self.driver.on_startup(self.start_forward)
        self.driver.on_shutdown(self.stop_forward)

    @override
    async def request(self, setup: Request) -> Response:
        try:
            response = await super().request(setup)
            if 200 <= response.status_code < 300:
                if not response.content:
                    raise ValueError("Empty response")
                return response
            elif response.status_code == 403:
                raise UnauthorizedException(response)
            elif response.status_code in (404, 405):
                raise ApiNotAvailable
            elif response.status_code == 429:
                raise RateLimitException(response)
            else:
                raise ActionFailed(response)
        except KaiheilaAdapterException:
            raise
        except Exception as e:
            raise NetworkError("API request failed") from e

    @override
    async def _call_api(self, bot: Bot, api: str, **data) -> Any:
        if isinstance(self.driver, ForwardDriver):
            if not self.api_root:
                raise ApiNotAvailable()

            match = re.findall(r"[A-Z]", api)
            if len(match) > 0:
                for m in match:
                    api = api.replace(m, "-" + m.lower())
            api = api.replace("_", "/")

            if api.startswith("/api/v3/"):
                api = api[len("/api/v3/") :]
            elif api.startswith("api/v3"):
                api = api[len("api/v3") :]
            api = api.strip("/")
            return await self._do_call_api(api, data, bot.token)

        else:
            raise ApiNotAvailable

    async def _do_call_api(
        self,
        api: str,
        data: Optional[Mapping[str, Any]] = None,
        token: Optional[str] = None,
    ) -> Any:
        log("DEBUG", f"Calling API <y>{api}</y>")
        data = dict(data) if data is not None else {}

        # 判断 POST 或 GET
        method = get_api_method(api) if not data.get("method") else data.get("method")

        headers = data.get("headers", {})

        files = None
        query = None
        body = None

        if "files" in data:
            files = data["files"]
            del data["files"]
        elif "file" in data:  # 目前只有asset/create接口需要上传文件（大概）
            files = {"file": data["file"]}
            del data["file"]

        if method == "GET":
            query = data
        elif method == "POST":
            body = data

        if token is not None:
            headers["Authorization"] = f"Bot {token}"

        request = Request(
            method,
            self.api_root + api,
            headers=headers,
            params=query,
            data=body,
            files=files,
            timeout=self.config.api_timeout,
        )
        result_type = get_api_restype(api)
        try:
            resp = await self.request(request)
            result = _handle_api_result(resp)
            return type_validate_python(result_type, result) if result_type else None
        except Exception as e:
            raise e

    async def _get_bot_info(self, token: str) -> User:
        return await self._do_call_api("user/me", token=token)

    async def _get_gateway(self, token: str) -> URL:
        result = await self._do_call_api(
            "gateway/index",
            data={"compress": 1 if self.kaiheila_config.compress else 0},
            token=token,
        )
        return result.url

    async def start_forward(self) -> None:
        for bot in self.kaiheila_config.kaiheila_bots:
            self.tasks.append(asyncio.create_task(self._forward_ws(bot)))

    async def stop_forward(self) -> None:
        for task in self.tasks:
            if not task.done():
                task.cancel()

        await asyncio.gather(
            *(asyncio.wait_for(task, timeout=10) for task in self.tasks),
            return_exceptions=True,
        )

    async def _forward_ws(self, bot_config: BotConfig) -> None:
        bot: Optional[Bot] = None

        heartbeat_task: Optional[asyncio.Task] = None

        while True:
            try:
                try:
                    url = URL(await self._get_gateway(bot_config.token))
                except TokenError as e:
                    log(
                        "ERROR",
                        f"<r><bg #f8bbd0>Token {escape_tag(bot_config.token)} was invalid. "
                        "Please get a new token from https://developer.kaiheila.cn/app/index </bg #f8bbd0></r>",
                        e,
                    )
                    return
                except Exception as e:
                    log(
                        "ERROR",
                        f"<r><bg #f8bbd0>Failed to get the Gateway URL for token {escape_tag(bot_config.token)}. "
                        "Trying to reconnect...</bg #f8bbd0></r>",
                        e,
                    )
                    continue

                headers = {}
                if bot_config.token:
                    headers["Authorization"] = f"Bot {bot_config.token}"
                request = Request("GET", url, headers=headers)

                async with self.websocket(request) as ws:
                    log(
                        "DEBUG",
                        f"WebSocket Connection to {escape_tag(str(url))} established",
                    )
                    try:
                        data_decompress_func = (
                            zlib.decompress
                            if self.kaiheila_config.compress
                            else lambda x: x
                        )
                        while True:
                            data = await ws.receive()
                            data = data_decompress_func(data)
                            json_data = json.loads(data)
                            event = self.json_to_event(
                                json_data,
                                bot and bot.self_id,
                                kaiheila_config=self.kaiheila_config,
                            )
                            if not event:
                                continue
                            if not bot:
                                if (
                                    not isinstance(event, LifecycleMetaEvent)
                                    or event.sub_type != "connect"
                                ):
                                    continue
                                bot_info = await self._get_bot_info(bot_config.token)
                                self_id = bot_info.id_
                                bot = Bot(
                                    self, self_id, bot_info.username, bot_config.token
                                )
                                self.connections[self_id] = ws
                                self.bot_connect(bot)

                                # start heartbeat
                                heartbeat_task = asyncio.create_task(
                                    self.start_heartbeat(bot)
                                )

                                log(
                                    "INFO",
                                    f"<y>Bot {escape_tag(self_id)}</y> connected",
                                )
                            asyncio.create_task(bot.handle_event(event))
                    except ReconnectError as e:
                        log(
                            "INFO",
                            "<r><bg #f8bbd0>Server requests reconnect"
                            f"{'for bot ' + escape_tag(bot.self_id) if bot else ''}.</bg #f8bbd0></r>",
                            e,
                        )
                    except TokenError:
                        raise
                    except Exception as e:
                        log(
                            "ERROR",
                            "<r><bg #f8bbd0>Error while process data from websocket"
                            f"{escape_tag(str(url))}. Trying to reconnect...</bg #f8bbd0></r>",
                            e,
                        )
                    finally:
                        if heartbeat_task:
                            heartbeat_task.cancel()
                            heartbeat_task = None

                        try:
                            await ws.close()
                        except:  # noqa: E722
                            pass

                        if bot:
                            # 清空本地的 sn 计数
                            ResultStore.set_sn(bot.self_id, 0)
                            self.connections.pop(bot.self_id, None)
                            self.bot_disconnect(bot)
                            bot = None
            except Exception as e:
                log(
                    "ERROR",
                    "<r><bg #f8bbd0>Error while setup websocket to "
                    f"{escape_tag(str(url))}. Trying to reconnect...</bg #f8bbd0></r>",
                    e,
                )

            await asyncio.sleep(RECONNECT_INTERVAL)

    async def start_heartbeat(self, bot: Bot) -> None:
        """
        每30s一次心跳
        :return:
        """
        while self.connections.get(bot.self_id):
            if self.connections.get(bot.self_id).closed:
                break
            try:
                await self.connections.get(bot.self_id).send(
                    json.dumps(
                        {
                            "s": 2,
                            "sn": ResultStore.get_sn(bot.self_id),  # 客户端目前收到的最新的消息 sn
                        }
                    )
                )
                await asyncio.sleep(26)
            except asyncio.CancelledError:
                raise
            except Exception as e:
                log(
                    "ERROR",
                    "<r><bg #f8bbd0>Error while sending heartbeat for bot"
                    f"{escape_tag(bot.self_id)}. Will retry after 1s ...</bg #f8bbd0></r>",
                    e,
                )
                await asyncio.sleep(1)

    @classmethod
    def json_to_event(
        cls,
        json_data: Any,
        self_id: Optional[str] = None,
        *,
        kaiheila_config: Optional[KaiheilaConfig] = None,
    ) -> Union[OriginEvent, Event, None]:
        if not isinstance(json_data, dict):
            return None

        signal = json_data["s"]

        if signal == SignalTypes.HELLO:
            if json_data["d"]["code"] == 0:
                data = json_data["d"]
                data["post_type"] = "meta_event"
                data["sub_type"] = "connect"
                data["meta_event_type"] = "lifecycle"
                return type_validate_python(LifecycleMetaEvent, data)
            elif json_data["d"]["code"] == 40103:
                raise ReconnectError
            elif json_data["d"]["code"] == 40101:
                raise TokenError("无效的 token")
            elif json_data["d"]["code"] == 40102:
                raise TokenError("token 验证失败")
        elif signal == SignalTypes.PONG:
            data = {"post_type": "meta_event", "meta_event_type": "heartbeat"}
            log(
                "TRACE",
                f"<y>Bot {escape_tag(str(self_id))}</y> HeartBeat",
            )
            return type_validate_python(HeartbeatMetaEvent, data)
        elif signal == SignalTypes.EVENT:
            ResultStore.set_sn(self_id, json_data["sn"])
        elif signal == SignalTypes.RECONNECT:
            raise ReconnectError
        elif signal == SignalTypes.RESUME_ACK:
            # 不存在的signal，resume是不可能resume的，这辈子都不会resume的，出了问题直接重连
            return

        # 屏蔽 Bot 自身
        if json_data["d"].get("author_id") == self_id :
            return
        # 屏蔽其他Bot消息
        if json_data["d"].get("extra", {}).get("author", {}).get("bot") and kaiheila_config.kaiheila_ignore_else_bots:
            return
        try:
            data = json_data["d"]
            extra = data.get("extra")

            data["self_id"] = self_id
            data["group_id"] = data.get("target_id")
            data["time"] = data.get("msg_timestamp")
            data["user_id"] = (
                data.get("author_id") if data.get("author_id") != "1" else "SYSTEM"
            )

            if data["type"] == EventTypes.sys:
                data["post_type"] = "notice"
                data["notice_type"] = extra.get("type")
                message = Message.template("{}").format(data["content"])
                data["message"] = message
                # data['notice_type'] = data.get('channel_type').lower()
                # data['notice_type'] = 'private' if data['notice_type'] == 'person' else data['notice_type']
            else:
                data["post_type"] = "message"
                data["sub_type"] = [
                    i.name.lower() for i in EventTypes if i.value == extra.get("type")
                ][0]
                data["message_type"] = data.get("channel_type").lower()
                data["message_type"] = (
                    "private"
                    if data["message_type"] == "person"
                    else data["message_type"]
                )
                data["extra"]["content"] = data.get("content")
                data["event"] = data["extra"]

            data["message_id"] = data.get("msg_id")
            post_type = data["post_type"]
            detail_type = data.get(f"{post_type}_type")
            detail_type = f".{detail_type}" if detail_type else ""
            sub_type = data.get("sub_type")
            sub_type = f".{sub_type}" if sub_type else ""

            event_name: str = post_type + detail_type + sub_type
            if kaiheila_config.ignore_events and event_name.startswith(kaiheila_config.ignore_events):
                return

            models = cls.get_event_model(event_name)
            for model in models:
                try:
                    event = type_validate_python(model, data)
                    break
                except Exception as e:
                    log("DEBUG", "Event Parser Error", e)
            else:
                event = type_validate_python(Event, json_data)
            log("DEBUG", escape_tag(str(model_dump(event))))
            return event
        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Failed to parse event. "
                f"Raw: {escape_tag(str(json_data))}</bg #f8bbd0></r>",
                e,
            )

    @classmethod
    def add_custom_model(cls, model: Type[Event]) -> None:
        if not model.__event__:
            raise ValueError("Event model's `__event__` attribute must be set")
        cls.event_models["." + model.__event__] = model

    @classmethod
    def get_event_model(cls, event_name: str) -> List[Type[Event]]:
        """
        :说明:

          根据事件名获取对应 ``Event Model`` 及 ``FallBack Event Model`` 列表, 不包括基类 ``Event``

        :返回:

          - ``List[Type[Event]]``
        """
        return [model.value for model in cls.event_models.prefixes("." + event_name)][
            ::-1
        ]

    @classmethod
    def custom_send(
        cls,
        send_func: Callable[[Bot, Event, Union[str, Message, MessageSegment]], None],
    ):
        setattr(Bot, "send_handler", send_func)



from nonebot.permission import Permission

from .event import ChannelMessageEvent, PrivateMessageEvent

# todo 增加更多权限


async def _private(event: PrivateMessageEvent) -> bool:
    return True


PRIVATE = Permission(_private)
"""
- **说明**: 匹配任意私聊消息类型事件
"""


async def _group(event: ChannelMessageEvent) -> bool:
    return True


GROUP = Permission(_group)
"""
- **说明**: 匹配任意群聊消息类型事件
"""

__all__ = ["PRIVATE", "GROUP"]



from typing import Literal, overload

from nonebot.compat import PYDANTIC_V2

__all__ = ("model_validator",)


if PYDANTIC_V2:
    from pydantic import model_validator as model_validator
else:
    from pydantic import root_validator

    @overload
    def model_validator(*, mode: Literal["before"]):
        ...

    @overload
    def model_validator(*, mode: Literal["after"]):
        ...

    def model_validator(*, mode: Literal["before", "after"]):
        return root_validator(pre=mode == "before", allow_reuse=True)



from pathlib import Path
from io import BytesIO, BufferedReader
from typing_extensions import override
from typing import TYPE_CHECKING, Any, Union, Literal, BinaryIO, Callable, Optional

from nonebot.message import handle_event

from nonebot.adapters import Bot as BaseBot

from .event import Event, MessageEvent
from .utils import log, escape_kmarkdown
from .api import ApiClient, MessageCreateReturn
from .message import (
    Text,
    Mention,
    Message,
    KMarkdown,
    MessageSegment,
    MessageSerializer,
)

if TYPE_CHECKING:
    from os import PathLike

    from .adapter import Adapter


def _check_at_me(bot: "Bot", event: MessageEvent):
    """
    :说明:

      检查消息开头或结尾是否存在 @机器人，去除并赋值 ``event.to_me``

    :参数:

      * ``bot: Bot``: Bot 对象
      * ``event: Event``: Event 对象
    """
    if not isinstance(event, MessageEvent):
        return

    # ensure message not empty
    if not event.message:
        event.message.append(MessageSegment.text(""))

    if event.message_type == "private":
        event.to_me = True
        return

    def _is_at_me_seg(seg: MessageSegment):
        return isinstance(seg, Mention) and seg.data["user_id"] == bot.self_id

    # check the first segment
    if _is_at_me_seg(event.message[0]):
        event.to_me = True
        event.message.pop(0)

        # 去除mention之后的空格
        if event.message and isinstance(event.message[0], Text):
            event.message[0].plain_text = event.message[0].plain_text.lstrip()
            if not event.message[0].plain_text:
                del event.message[0]

    if not event.to_me:
        # check the last segment
        i = -1
        last_msg_seg = event.message[i]
        if (
            isinstance(last_msg_seg, Text)
            and not last_msg_seg.plain_text.strip()
            and len(event.message) >= 2
        ):
            i -= 1
            last_msg_seg = event.message[i]

        if _is_at_me_seg(last_msg_seg):
            event.to_me = True
            del event.message[i:]

    # 避免消息为空
    if event.message == 0:
        event.message.append(MessageSegment.text(""))


def _check_nickname(bot: "Bot", event: MessageEvent):
    """
    :说明:

      检查消息开头是否存在昵称，去除并赋值 ``event.to_me``

    :参数:

      * ``bot: Bot``: Bot 对象
      * ``event: Event``: Event 对象
    """
    nicknames = set(filter(lambda n: n, bot.config.nickname))
    first_seg = event.message[0]

    if isinstance(first_seg, Text):
        first_text = first_seg.data["text"]

        for nickname in nicknames:
            if first_text.startswith(nickname):
                log("DEBUG", f"User is calling me {nickname}")
                event.to_me = True
                first_seg.data["text"] = first_text.removeprefix(nickname)
                break
    elif isinstance(first_seg, KMarkdown):
        first_text = first_seg.data["raw_content"]

        for nickname in nicknames:
            if first_text.startswith(nickname):
                log("DEBUG", f"User is calling me {nickname}")
                event.to_me = True
                first_seg.data["raw_content"] = first_text.removeprefix(nickname)
                first_seg.data["content"] = first_seg.data["content"].removeprefix(
                    escape_kmarkdown(nickname)
                )
                break


async def send(
    bot: "Bot",
    event: Event,
    message: Union[str, Message, MessageSegment],
    reply_sender: bool = False,
    is_temp_msg: bool = False,
    **kwargs: Any,
) -> Any:
    # 构造参数
    params = {**kwargs}

    # quote
    if reply_sender:
        params.setdefault("quote", getattr(event, "message_id"))

    # message_type
    if event.channel_type == "GROUP":
        params.setdefault("message_type", "channel")

        # temp_target_id
        if is_temp_msg:
            params.setdefault("user_id", getattr(event, "user_id"))

        # target_id
        if getattr(event, "target_id", None):
            params.setdefault("channel_id", getattr(event, "target_id"))
    else:
        params.setdefault("message_type", "private")

        # target_id
        if getattr(event, "target_id", None):
            params.setdefault("user_id", getattr(event, "user_id"))

    params.setdefault("message", message)

    return await bot.send_msg(**params)


class Bot(BaseBot, ApiClient):
    """
    Kaiheila Bot 适配。
    """

    send_handler: Callable[
        ["Bot", Event, Union[str, Message, MessageSegment], bool, bool], Any
    ] = send

    def __init__(self, adapter: "Adapter", self_id: str, name: str, token: str):
        """
        :参数:

          * ``adapter: Adapter``: 适配器
          * ``self_id: str``: 机器人 ID
          * ``name: str``: 机器人用户名
          * ``token``: 机器人 token

        """
        super().__init__(adapter, self_id)
        self.self_name: str = name
        self.token: str = token

    @override
    async def call_api(self, api: str, **data) -> Any:
        """
        :说明:

          调用 kaiheila 协议 API

        :参数:

          * ``api: str``: API 名称
          * ``**data: Any``: API 参数

        :返回:

          - ``Any``: API 调用返回数据

        :异常:

          - ``NetworkError``: 网络错误
          - ``ActionFailed``: API 调用失败
        """
        return await super().call_api(api, **data)

    async def handle_event(self, event: Event) -> None:
        if isinstance(event, MessageEvent):
            _check_at_me(self, event)
            _check_nickname(self, event)
            pass

        await handle_event(self, event)

    @override
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        reply_sender: bool = False,
        is_temp_msg: bool = False,
        **kwargs,
    ) -> MessageCreateReturn:
        """
        :说明:

          根据 ``event``  向触发事件的主体发送消息。

        :参数:

          * ``event: Event``: Event 对象
          * ``message: Union[str, Message, MessageSegment]``: 要发送的消息
          * ``reply_sender: bool``: 是否回复原消息
          * ``is_temp_msg: bool``: 是否临时消息
          * ``**kwargs``: 覆盖默认参数

        :返回:

          - ``Any``: API 调用返回数据

        :异常:

          - ``ValueError``: 缺少 ``user_id``, ``channel_id``
          - ``NetworkError``: 网络错误
          - ``ActionFailed``: API 调用失败
        """
        return await self.__class__.send_handler(
            self, event, message, reply_sender, is_temp_msg, **kwargs
        )

    async def send_private_msg(
        self,
        *,
        user_id: str,
        message: Union[str, Message, MessageSegment],
        quote: Optional[str] = None,
    ) -> MessageCreateReturn:
        """发送私聊消息。

        user_id: 对方用户ID
        message: 要发送的内容，字符串类型将作为纯文本消息发送
        quote: 回复某条消息的消息ID
        """
        return await self.send_msg(
            message_type="private", user_id=user_id, message=message, quote=quote
        )

    async def send_channel_msg(
        self,
        *,
        channel_id: str,
        message: Union[str, Message, MessageSegment],
        quote: Optional[str] = None,
    ) -> MessageCreateReturn:
        """发送频道消息。

        channel_id: 频道ID
        message: 要发送的内容，字符串类型将作为纯文本消息发送
        quote: 回复某条消息的消息ID
        """
        return await self.send_msg(
            message_type="channel", channel_id=channel_id, message=message, quote=quote
        )

    async def send_temp_msg(
        self,
        *,
        user_id: str,
        channel_id: str,
        message: Union[str, Message, MessageSegment],
        quote: Optional[str] = None,
    ) -> MessageCreateReturn:
        """发送频道临时消息。该消息不会存数据库，但是会在频道内只给该用户推送临时消息。用于在频道内针对用户的操作进行单独的回应通知等。

        channel_id: 频道ID
        message: 要发送的内容，字符串类型将作为纯文本消息发送
        quote: 回复某条消息的消息ID
        """
        return await self.send_msg(
            message_type="temp",
            user_id=user_id,
            channel_id=channel_id,
            message=message,
            quote=quote,
        )

    async def send_msg(
        self,
        *,
        message_type: Literal["private", "channel", "temp", ""] = "",
        user_id: Optional[str] = None,
        channel_id: Optional[str] = None,
        message: Union[str, Message, MessageSegment],
        quote: Optional[str] = None,
    ) -> MessageCreateReturn:
        """发送消息。

        参数:
            message_type: 消息类型，支持 `private`、`channel`、`temp`，分别对应私聊、频道，如不传入，则根据传入的 `*_id` 参数判断
            user_id: 对方用户ID（消息类型为 `private`、`temp` 时需要）
            channel_id: 频道ID（消息类型为 `channel`、`temp` 时需要）
            message: 要发送的内容，字符串类型将作为纯文本消息发送
            quote: 回复某条消息的消息ID
        """
        # 接口文档：
        # https://developer.kaiheila.cn/doc/http/direct-message#%E5%8F%91%E9%80%81%E7%A7%81%E4%BF%A1%E8%81%8A%E5%A4%A9%E6%B6%88%E6%81%AF
        # https://developer.kaiheila.cn/doc/http/message#%E5%8F%91%E9%80%81%E9%A2%91%E9%81%93%E8%81%8A%E5%A4%A9%E6%B6%88%E6%81%AF
        params = {}

        # type & content
        if isinstance(message, Message):
            serialized_data = await MessageSerializer(message).serialize(self)
        else:
            serialized_data = await MessageSerializer(Message(message)).serialize(self)
        params = {**params, **serialized_data}

        # quote
        if quote is not None:
            params["quote"] = quote

        # target_id & api
        if message_type == "channel":
            params["target_id"] = channel_id
            api = "message_create"
        elif message_type == "private":
            params["target_id"] = user_id
            api = "directMessage_create"
        elif message_type == "temp":
            params["target_id"] = channel_id
            params["temp_target_id"] = user_id
            api = "message_create"
        else:
            if channel_id and not user_id:
                params["target_id"] = channel_id
                api = "message_create"
            elif not channel_id and user_id:
                params["target_id"] = user_id
                api = "directMessage_create"
            elif channel_id and user_id:
                params["target_id"] = channel_id
                params["temp_target_id"] = user_id
                api = "message_create"
            else:
                raise ValueError("channel_id 和 user_id 不能同时为 None")

        return await self.call_api(api, **params)

    async def upload_file(
        self,
        file: Union[str, "PathLike[str]", BinaryIO, bytes],
        filename: Optional[str] = None,
    ) -> str:
        """
        上传文件。

        参数:
            file: 文件，可以是文件路径（str, PathLike[str]）、打开的文件流（BinaryIO）、或二进制数据（bytes）
            filename: 文件名

        返回值:
            文件的 URL
        """
        if isinstance(file, BytesIO):
            file = file.getvalue()
        elif isinstance(file, BufferedReader):
            file = file.read()
        elif isinstance(file, str) or isinstance(file, Path):
            with open(file, "rb") as img:
                file = img.read()
        # 经过测试，服务器会用从文件读取到的mime覆盖掉我们传过去的mime
        file = (filename or "upload-file", file, "application/octet-stream")
        result = await self.asset_create(file=file)
        return result.url



from typing import Any, Dict, List, Union, Optional

from pydantic import Field, BaseModel


class User(BaseModel):
    """
    开黑啦 User 字段

    https://developer.kaiheila.cn/doc/objects
    """

    id_: Optional[str] = Field(None, alias="id")
    """用户的 id"""
    username: Optional[str] = None
    """用户的名称"""
    nickname: Optional[str] = None
    """用户在当前服务器的昵称"""
    identify_num: Optional[str] = None
    """用户名的认证数字，用户名正常为：user_name#identify_num"""
    online: Optional[bool] = None
    """当前是否在线"""
    bot: Optional[bool] = None
    """是否为机器人"""
    os: Optional[str] = None
    """os"""
    status: Optional[int] = None
    """用户的状态,`0`和`1`代表正常，`10`代表被封禁"""
    avatar: Optional[str] = None
    """用户的头像的 url 地址"""
    vip_avatar: Optional[str] = None
    """	vip 用户的头像的 url 地址，可能为 gif 动图"""
    mobile_verified: Optional[bool] = None
    """是否手机号已验证"""
    roles: Optional[List[int]] = None
    """用户在当前服务器中的角色 id 组成的列表"""
    joined_at: Optional[int] = None
    """加入服务器时间"""
    active_time: Optional[int] = None
    """上次在线时间"""


class Role(BaseModel):
    """角色"""

    role_id: Optional[int] = None
    """角色 id"""
    name: Optional[str] = None
    """角色名称"""
    color: Optional[int] = None
    """颜色色值"""
    position: Optional[int] = None
    """顺序位置"""
    hoist: Optional[int] = None
    """是否为角色设定(与普通成员分开显示)"""
    mentionable: Optional[int] = None
    """是否允许任何人@提及此角色"""
    permissions: Optional[int] = None
    """权限码"""


class PermissionOverwrite(BaseModel):
    role_id: Optional[int] = None
    allow: Optional[int] = None
    deny: Optional[int] = None


class PermissionUser(BaseModel):
    user: Optional[User] = None
    allow: Optional[int] = None
    deny: Optional[int] = None


class ChannelRoleInfo(BaseModel):
    """频道角色权限详情"""

    permission_overwrites: Optional[List[PermissionOverwrite]] = None
    """针对角色在该频道的权限覆写规则组成的列表"""
    permission_users: Optional[List[PermissionUser]] = None
    """针对用户在该频道的权限覆写规则组成的列表"""
    permission_sync: Optional[int] = None
    """权限设置是否与分组同步, 1 or 0"""


class ChannelRoleSyncResult(BaseModel):
    """同步频道角色权限结果"""

    permission_overwrites: Optional[List[PermissionOverwrite]] = None
    """针对角色在该频道的权限覆写规则组成的列表"""
    permission_users: Optional[List[PermissionUser]] = None
    """针对用户在该频道的权限覆写规则组成的列表"""


class Channel(ChannelRoleInfo):
    """开黑啦 频道 字段"""

    id_: Optional[str] = Field(None, alias="id")
    """频道 id"""
    name: Optional[str] = None
    """频道名称"""
    user_id: Optional[str] = None
    """创建者 id"""
    master_id: Optional[str] = None
    """master id """
    guild_id: Optional[str] = None
    """服务器 id"""
    topic: Optional[str] = None
    """频道简介"""
    is_category: Optional[bool] = None
    """是否为分组，事件中为 int 格式"""
    parent_id: Optional[str] = None
    """上级分组的 id"""
    level: Optional[int] = None
    """排序 level"""
    slow_mode: Optional[int] = None
    """慢速模式下限制发言的最短时间间隔, 单位为秒(s)"""
    type: Optional[int] = None
    """频道类型: 1 文字频道, 2 语音频道"""
    has_password: Optional[bool] = None
    """是否有密码"""
    limit_amount: Optional[int] = None
    """人数限制"""


class Guild(BaseModel):
    """服务器"""

    id_: Optional[str] = Field(None, alias="id")
    """服务器 id"""
    name: Optional[str] = None
    """服务器名称"""
    topic: Optional[str] = None
    """服务器主题"""
    user_id: Optional[str] = None
    """服务器主的 id"""
    icon: Optional[str] = None
    """服务器 icon 的地址"""
    notify_type: Optional[int] = None
    """通知类型\n
    `0`代表默认使用服务器通知设置\n
    `1`代表接收所有通知\n
    `2`代表仅@被提及\n
    `3`代表不接收通知
    """
    region: Optional[str] = None
    """服务器默认使用语音区域"""
    enable_open: Optional[bool] = None
    """是否为公开服务器"""
    open_id: Optional[str] = None
    """公开服务器 id"""
    default_channel_id: Optional[str] = None
    """默认频道 id"""
    welcome_channel_id: Optional[str] = None
    """欢迎频道 id"""
    roles: Optional[List[Role]] = None
    """角色列表"""
    channels: Optional[List[Channel]] = None
    """频道列表"""


class Quote(BaseModel):
    """引用消息"""

    id_: Optional[str] = Field(None, alias="id")
    """引用消息 id"""
    type: Optional[int] = None
    """引用消息类型"""
    content: Optional[str] = None
    """引用消息内容"""
    create_at: Optional[int] = None
    """引用消息创建时间（毫秒）"""
    author: Optional[User] = None
    """作者的用户信息"""


class Attachments(BaseModel):
    """附加的多媒体数据"""

    type: Optional[str] = None
    """多媒体类型"""
    url: Optional[str] = None
    """多媒体地址"""
    name: Optional[str] = None
    """多媒体名"""
    size: Optional[int] = None
    """大小 单位（B）"""


class Emoji(BaseModel):
    id_: Optional[str] = Field(None, alias="id")
    name: Optional[str] = None

    # 转义 unicdoe 为 emoji表情
    # @root_validator(pre=True)
    # def parse_emoji(cls, values: dict):
    #     values['id'] = chr(int(values['id'][2:-2]))
    #     values['name'] = chr(int(values['name'][2:-2]))
    #     return values


class URL(BaseModel):
    url: Optional[str] = None
    """资源的 url"""


class Meta(BaseModel):
    page: Optional[int] = None
    page_total: Optional[int] = None
    page_size: Optional[int] = None
    total: Optional[int] = None


class ListReturn(BaseModel):
    meta: Optional[Meta] = None
    sort: Optional[Dict[str, Any]] = None


class BlackList(BaseModel):
    """黑名单"""

    user_id: Optional[str] = None
    """用户 id"""
    created_time: Optional[int] = None
    """加入黑名单的时间戳(毫秒)"""
    remark: Optional[str] = None
    """加入黑名单的原因"""
    user: Optional[User] = None
    """用户"""


class BlackListsReturn(ListReturn):
    """获取黑名单列表返回信息"""

    blacklists: Optional[List[BlackList]] = Field(None, alias="items")
    """黑名单列表"""


class MessageCreateReturn(BaseModel):
    """发送频道消息返回信息"""

    msg_id: Optional[str] = None
    """服务端生成的消息 id"""
    msg_timestamp: Optional[int] = None
    """消息发送时间(服务器时间戳)"""
    nonce: Optional[str] = None
    """随机字符串"""


class ChannelRoleReturn(BaseModel):
    """创建或更新频道角色权限返回信息"""

    role_id: Optional[int] = None
    user_id: Optional[str] = None
    allow: Optional[int] = None
    deny: Optional[int] = None


class GetUserJoinedChannelReturn(ListReturn):
    channels: Optional[List[Channel]] = Field(None, alias="items")


class GuildsReturn(ListReturn):
    guilds: Optional[List[Guild]] = Field(None, alias="items")


class ChannelsReturn(ListReturn):
    channels: Optional[List[Channel]] = Field(None, alias="items")


class GuildUsersReturn(ListReturn):
    """服务器中的用户列表"""

    users: Optional[List[User]] = Field(None, alias="items")
    """用户列表"""
    user_count: Optional[int] = None
    """用户数量"""
    online_count: Optional[int] = None
    """在线用户数量"""
    offline_count: Optional[int] = None
    """离线用户数量"""


class Reaction(BaseModel):
    emoji: Optional[Emoji] = None
    count: Optional[int] = None
    me: Optional[bool] = None


class MentionInfo(BaseModel):
    mention_part: Optional[List[dict]] = None
    mention_role_part: Optional[List[dict]] = None
    channel_part: Optional[List[dict]] = None
    item_part: Optional[List[dict]] = None


class BaseMessage(BaseModel):
    id_: Optional[str] = Field(None, alias="id")
    """消息 ID"""
    type: Optional[int] = None
    """消息类型"""
    content: Optional[str] = None
    """消息内容"""
    embeds: Optional[List[dict]] = None
    """超链接解析数据"""
    attachments: Optional[Union[bool, Attachments]] = None
    """附加的多媒体数据"""
    create_at: Optional[int] = None
    """创建时间"""
    updated_at: Optional[int] = None
    """更新时间"""
    reactions: Optional[List[Reaction]] = None
    """回应数据"""
    image_name: Optional[str] = None
    """"""
    read_status: Optional[bool] = None
    """是否已读"""
    quote: Optional[Quote] = None
    """引用数据"""
    mention_info: Optional[MentionInfo] = None
    """引用特定用户或特定角色的信息"""


class ChannelMessage(BaseMessage):
    """频道消息"""

    author: Optional[User] = None
    mention: Optional[List[Any]] = None
    mention_all: Optional[bool] = None
    mention_roles: Optional[List[Any]] = None
    mention_here: Optional[bool] = None


class DirectMessage(BaseMessage):
    """私聊消息"""

    author_id: Optional[str] = None
    """作者的用户 ID"""
    from_type: Optional[int] = None
    """from_type"""
    msg_icon: Optional[str] = None
    """msg_icon"""


class ChannelMessagesReturn(BaseModel):
    """获取私信聊天消息列表返回信息"""

    direct_messages: Optional[List[ChannelMessage]] = Field(None, alias="items")


class DirectMessagesReturn(BaseModel):
    """获取私信聊天消息列表返回信息"""

    direct_messages: Optional[List[DirectMessage]] = Field(None, alias="items")


class ReactionUser(User):
    reaction_time: Optional[int] = None


class TargetInfo(BaseModel):
    """私聊会话 目标用户信息"""

    id_: Optional[str] = Field(None, alias="id")
    """目标用户 ID"""
    username: Optional[str] = None
    """目标用户名"""
    online: Optional[bool] = None
    """是否在线"""
    avatar: Optional[str] = None
    """头像图片链接"""


class UserChat(BaseModel):
    """私聊会话"""

    code: Optional[str] = None
    """私信会话 Code"""
    last_read_time: Optional[int] = None
    """上次阅读消息的时间 (毫秒)"""
    latest_msg_time: Optional[int] = None
    """最新消息时间 (毫秒)"""
    unread_count: Optional[int] = None
    """未读消息数"""
    target_info: Optional[TargetInfo] = None
    """目标用户信息"""


class UserChatsReturn(ListReturn):
    """获取私信聊天会话列表返回信息"""

    user_chats: Optional[List[UserChat]] = Field(None, alias="items")
    """私聊会话列表"""


class RolesReturn(ListReturn):
    """获取服务器角色列表返回信息"""

    roles: Optional[List[Role]] = Field(None, alias="items")
    """服务器角色列表"""


class GuildRoleReturn(BaseModel):
    """赋予或删除用户角色返回信息"""

    user_id: Optional[str] = None
    """用户 id"""
    guild_id: Optional[str] = None
    """服务器 id"""
    roles: Optional[List[int]] = None
    """角色 id 的列表"""


class IntimacyImg(BaseModel):
    """形象图片的总列表"""

    id_: Optional[str] = Field(None, alias="id")
    """	形象图片的 id"""
    url: Optional[str] = None
    """形象图片的地址"""


class IntimacyIndexReturn(BaseModel):
    """获取用户亲密度返回信息"""

    img_url: Optional[str] = None
    """机器人给用户显示的形象图片地址"""
    social_info: Optional[str] = None
    """机器人显示给用户的社交信息"""
    last_read: Optional[int] = None
    """用户上次查看的时间戳"""
    score: Optional[int] = None
    """亲密度，0-2200"""
    img_list: Optional[List[IntimacyImg]] = None
    """形象图片的总列表"""


class GuildBoost(BaseModel):
    """服务器助力历史"""

    user_id: Optional[str] = None
    """使用助力包的用户 ID"""
    guild_id: Optional[str] = None
    """服务器的 ID"""
    start_time: Optional[int] = None
    """助力包生效时间, Unix 时间戳 (单位: 秒)"""
    end_time: Optional[int] = None
    """助力包失效时间, Unix 时间戳 (单位: 秒)"""
    user: Optional[User] = None
    """使用助力包的用户数据对象"""


class GuildBoostReturn(ListReturn):
    """获取服务器助力历史返回信息"""

    boost: Optional[List[GuildBoost]] = Field(None, alias="items")
    """服务器助力历史列表"""


class GuildEmoji(BaseModel):
    """服务器表情"""

    name: Optional[str] = None
    """表情的名称"""
    id_: Optional[str] = Field(None, alias="id")
    """表情的 ID"""
    user_info: Optional[User] = None
    """上传用户"""


class GuildEmojisReturn(ListReturn):
    """获取服务器表情列表返回信息"""

    roles: Optional[List[GuildEmoji]] = Field(None, alias="items")
    """服务器表情列表"""


class Invite(BaseModel):
    """邀请信息"""

    guild_id: Optional[str] = None
    """服务器 id"""
    channel_id: Optional[str] = None
    """频道 id"""
    url_code: Optional[str] = None
    """url code"""
    url: Optional[str] = None
    """地址"""
    user: Optional[User] = None
    """用户"""


class InvitesReturn(ListReturn):
    """获取邀请列表返回信息"""

    roles: Optional[List[Invite]] = Field(None, alias="items")
    """邀请列表"""


__all__ = (
    "User",
    "Role",
    "PermissionOverwrite",
    "PermissionUser",
    "ChannelRoleInfo",
    "ChannelRoleSyncResult",
    "Channel",
    "Guild",
    "Quote",
    "Attachments",
    "Emoji",
    "URL",
    "Meta",
    "ListReturn",
    "BlackList",
    "BlackListsReturn",
    "MessageCreateReturn",
    "ChannelRoleReturn",
    "GetUserJoinedChannelReturn",
    "GuildsReturn",
    "ChannelsReturn",
    "GuildUsersReturn",
    "Reaction",
    "MentionInfo",
    "BaseMessage",
    "ChannelMessage",
    "DirectMessage",
    "ChannelMessagesReturn",
    "DirectMessagesReturn",
    "ReactionUser",
    "TargetInfo",
    "UserChat",
    "UserChatsReturn",
    "RolesReturn",
    "GuildRoleReturn",
    "IntimacyImg",
    "IntimacyIndexReturn",
    "GuildBoost",
    "GuildBoostReturn",
    "GuildEmoji",
    "GuildEmojisReturn",
    "Invite",
    "InvitesReturn",
)



import json
import asyncio
from io import StringIO
from collections import UserDict
from typing import Any, Dict, Tuple, Optional, Protocol, runtime_checkable

from nonebot.utils import logger_wrapper
from nonebot.internal.driver import Response

from .exception import ActionFailed

log = logger_wrapper("Kaiheila")


def _b2s(b: Optional[bool]) -> Optional[str]:
    """转换布尔值为字符串。"""
    return b if b is None else str(b).lower()


def code_to_emoji(emoji: str) -> bytes:
    return emoji.encode("unicode_escape")


ESCAPE_CHAR = "!()*-.:>[\\]`~"


def escape_kmarkdown(content: str):
    """
    将文本中的kmarkdown标识符进行转义
    """
    with StringIO() as f:
        for c in content:
            if c in ESCAPE_CHAR:
                f.write("\\")
            f.write(c)
        return f.getvalue()


def unescape_kmarkdown(content: str):
    """
    去除kmarkdown中的转义字符
    """
    with StringIO() as f:
        i = 0
        while i < len(content):
            if content[i] == "\\":
                if i + 1 < len(content) and content[i + 1] in ESCAPE_CHAR:
                    f.write(content[i + 1])
                    i += 2
                    continue

            f.write(content[i])
            i += 1
        return f.getvalue()


def _handle_api_result(response: Response) -> Any:
    """
    :说明:

      处理 API 请求返回值。

    :参数:

      * ``response: Response``: API 响应体

    :返回:

        - ``T``: API 调用返回数据

    :异常:

        - ``ActionFailed``: API 调用失败
    """
    result = json.loads(response.content)
    if isinstance(result, dict):
        log("DEBUG", "API result " + str(result))
        if result.get("code") != 0:
            raise ActionFailed(response)
        else:
            return result.get("data")


class ResultStore:
    _seq = 1
    _futures: Dict[Tuple[str, int], asyncio.Future] = {}
    _sn_map = {}

    @classmethod
    def set_sn(cls, self_id: str, sn: int) -> None:
        cls._sn_map[self_id] = sn

    @classmethod
    def get_sn(cls, self_id: str) -> int:
        return cls._sn_map.get(self_id, 0)


class AttrDict(UserDict):
    def __init__(self, data=None):
        initial = dict(data)
        for k in initial:
            if isinstance(initial[k], dict):
                initial[k] = AttrDict(initial[k])

        super().__init__(initial)

    def __getattr__(self, name):
        return self[name]


@runtime_checkable
class BytesReadable(Protocol):
    def read(self) -> bytes:
        ...
